@@ -46,8 +46,7 @@ A very brief overview of the language contents follows:
 
     Cell, ProductCell, OuterProductCell,
     interval, triangle, tetrahedron,
-    quadrilateral, hexahedron,
-    cell1D, cell2D, cell3D,
+    quadrilateral, hexahedron
 
 * Sobolev spaces::
 
@@ -60,6 +59,8 @@ A very brief overview of the language contents follows:
     EnrichedElement, RestrictedElement,
     TensorProductElement, OuterProductElement,
     OuterProductVectorElement, HDiv, HCurl
+    BrokenElement, TraceElement
+    FacetElement, InteriorElement
 
 * Arguments::
 
@@ -81,7 +82,7 @@ A very brief overview of the language contents follows:
 
     SpatialCoordinate,
     FacetNormal, CellNormal,
-    CellVolume, Circumradius,
+    CellVolume, Circumradius, MinCellEdgeLength, MaxCellEdgeLength,
     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
 
 * Indices::
@@ -117,7 +118,7 @@ A very brief overview of the language contents follows:
 
 * Nonlinear functions::
 
-    Max, Min,
+    max_value, min_value,
     abs, sign, sqrt,
     exp, ln, erf,
     cos, sin, tan,
@@ -137,7 +138,9 @@ A very brief overview of the language contents follows:
 
 * Integral measures::
 
-    dx, ds, ds_b, ds_t, ds_tb, ds_v, dS, dS_h, dS_v, dP, dc, dE
+    dx, ds, dS, dP,
+    dc, dC, dO, dI,
+    ds_b, ds_t, ds_tb, ds_v, dS_h, dS_v
 
 * Form transformations::
 
@@ -170,7 +173,7 @@ A very brief overview of the language contents follows:
 # Modified by Andrew T. T. McRae, 2014
 # Modified by Lawrence Mitchell, 2014
 
-__version__ = "1.4.0"
+__version__ = "1.6.0"
 
 ########## README
 # Imports here should be what the user sees when doing "from ufl import *",
@@ -193,7 +196,7 @@ from ufl.domain import as_domain, Domain, ProductDomain
 from ufl.geometry import (
     SpatialCoordinate,
     FacetNormal, CellNormal,
-    CellVolume, Circumradius,
+    CellVolume, Circumradius, MinCellEdgeLength, MaxCellEdgeLength,
     FacetArea, MinFacetEdgeLength, MaxFacetEdgeLength,
     )
 
@@ -204,7 +207,8 @@ from ufl.sobolevspace import L2, H1, H2, HDiv, HCurl
 from ufl.finiteelement import FiniteElementBase, FiniteElement, \
     MixedElement, VectorElement, TensorElement, EnrichedElement, \
     RestrictedElement, TensorProductElement, OuterProductElement, \
-    OuterProductVectorElement, HDiv, HCurl
+    OuterProductVectorElement, HDiv, HCurl, BrokenElement, TraceElement, \
+    FacetElement, InteriorElement
 
 # Hook to extend predefined element families
 from ufl.finiteelement.elementlist import register_element, show_elements #, ufl_elements
@@ -224,7 +228,7 @@ from ufl.split_functions import split
 from ufl.constantvalue import PermutationSymbol, Identity, zero, as_ufl
 
 # Indexing of tensor expressions
-from ufl.indexing import Index, indices
+from ufl.core.multiindex import Index, indices
 
 # Special functions for expression base classes
 # (ensure this is imported, since it attaches operators to Expr)
@@ -246,7 +250,7 @@ from ufl.operators import rank, shape, \
                        cosh, sinh, tanh, \
                        bessel_J, bessel_Y, bessel_I, bessel_K, \
                        eq, ne, le, ge, lt, gt, And, Or, Not, \
-                       conditional, sign, Max, Min, \
+                       conditional, sign, max_value, min_value, Max, Min, \
                        variable, diff, \
                        Dx,  grad, div, curl, rot, nabla_grad, nabla_div, Dn, exterior_derivative, \
                        jump, avg, cell_avg, facet_avg, \
@@ -270,11 +274,14 @@ from ufl.formoperators import replace, derivative, action, energy_norm, rhs, lhs
     system, functional, adjoint, sensitivity_rhs #, dirichlet_functional
 
 # Predefined convenience objects
-from ufl.objects import \
-    vertex, interval, triangle, tetrahedron, \
-    quadrilateral, hexahedron, facet, cell1D, cell2D, cell3D, \
-    i, j, k, l, p, q, r, s, \
-    dx, ds, ds_b, ds_t, ds_tb, ds_v, dS, dS_h, dS_v, dP, dc, dE
+from ufl.objects import (
+    vertex, interval, triangle, tetrahedron,
+    quadrilateral, hexahedron, facet,
+    i, j, k, l, p, q, r, s,
+    dx, ds, dS, dP,
+    dc, dC, dO, dI,
+    ds_b, ds_t, ds_tb, ds_v, dS_h, dS_v
+    )
 
 # Useful constants
 from math import e, pi
@@ -287,13 +294,14 @@ __all__ = [
     'as_domain', 'Domain', 'ProductDomain',
     'L2', 'H1', 'H2', 'HCurl', 'HDiv',
     'SpatialCoordinate',
-    'CellVolume', 'Circumradius',
+    'CellVolume', 'Circumradius', 'MinCellEdgeLength', 'MaxCellEdgeLength',
     'FacetArea', 'MinFacetEdgeLength', 'MaxFacetEdgeLength',
     'FacetNormal', 'CellNormal',
     'FiniteElementBase', 'FiniteElement',
     'MixedElement', 'VectorElement', 'TensorElement', 'EnrichedElement',
     'RestrictedElement', 'TensorProductElement', 'OuterProductElement',
     'OuterProductVectorElement', 'HDiv', 'HCurl',
+    'BrokenElement', 'TraceElement', 'FacetElement', 'InteriorElement',
     'register_element', 'show_elements',
     'Argument', 'TestFunction', 'TrialFunction',
     'Arguments', 'TestFunctions', 'TrialFunctions',
@@ -314,7 +322,7 @@ __all__ = [
     'cosh', 'sinh', 'tanh',
     'bessel_J', 'bessel_Y', 'bessel_I', 'bessel_K',
     'eq', 'ne', 'le', 'ge', 'lt', 'gt', 'And', 'Or', 'Not',
-    'conditional', 'sign', 'Max', 'Min',
+    'conditional', 'sign', 'max_value', 'min_value', 'Max', 'Min',
     'variable', 'diff',
     'Dx', 'grad', 'div', 'curl', 'rot', 'nabla_grad', 'nabla_div', 'Dn', 'exterior_derivative',
     'jump', 'avg', 'cell_avg', 'facet_avg',
@@ -323,10 +331,11 @@ __all__ = [
     'Integral', 'Measure', 'register_integral_type', 'integral_types',
     'replace', 'replace_integral_domains', 'derivative', 'action', 'energy_norm', 'rhs', 'lhs',
     'system', 'functional', 'adjoint', 'sensitivity_rhs',
-    'dx', 'ds', 'ds_b', 'ds_t', 'ds_tb', 'ds_v', 'dS', 'dS_h', 'dS_v', 'dP', 'dc', 'dE',
+    'dx', 'ds', 'dS', 'dP',
+    'dc', 'dC', 'dO', 'dI',
+    'ds_b', 'ds_t', 'ds_tb', 'ds_v', 'dS_h', 'dS_v',
     'vertex', 'interval', 'triangle', 'tetrahedron',
     'quadrilateral', 'hexahedron', 'facet',
-    'cell1D', 'cell2D', 'cell3D',
     'i', 'j', 'k', 'l', 'p', 'q', 'r', 's',
     'e', 'pi',
     ]
