@@ -19,28 +19,32 @@
 #
 # Modified by Anders Logg, 2008-2009.
 
+
 # Utilities for traversing over expression trees in different ways
-from ufl.algorithms.traversal import iter_expressions, \
-                                     traverse_terminals, traverse_unique_terminals, \
-                                     post_traversal, pre_traversal, \
-                                     post_walk, pre_walk, walk
+from ufl.algorithms.traversal import iter_expressions
 
-# Class for simple extraction of form meta data
-from ufl.algorithms.formdata import FormData
+# Keeping these imports here for backwards compatibility, doesn't cost anything.
+# Prefer importing from ufl.corealg.traversal in future code.
+from ufl.corealg.traversal import pre_traversal, post_traversal
+from ufl.corealg.traversal import traverse_terminals, traverse_unique_terminals
 
-# Function for preprocessing a form
-from ufl.algorithms.preprocess import preprocess, preprocess_expression
 
 # Utilities for extracting information from forms and expressions
-from ufl.algorithms.analysis import extract_classes, extract_type, has_type, \
-                                    extract_arguments, extract_coefficients, \
-                                    extract_arguments_and_coefficients, \
-                                    extract_elements, extract_unique_elements, \
-                                    extract_sub_elements, extract_unique_sub_elements, \
-                                    extract_variables, extract_duplications, \
-                                    extract_max_quadrature_element_degree, \
-                                    estimate_quadrature_degree, \
-                                    sort_elements
+from ufl.algorithms.analysis import (
+    extract_type,
+    extract_arguments,
+    extract_coefficients,
+    extract_arguments_and_coefficients,
+    extract_elements,
+    extract_unique_elements,
+    extract_sub_elements,
+    sort_elements,
+    )
+
+
+# Preprocessing a form to extract various meta data
+from ufl.algorithms.formdata import FormData
+from ufl.algorithms.compute_form_data import compute_form_data
 
 # Utilities for checking properties of forms
 from ufl.algorithms.predicates import is_multilinear
@@ -50,9 +54,9 @@ from ufl.algorithms.signature import compute_form_signature
 from ufl.algorithms.checks import validate_form
 
 # Utilites for modifying expressions and forms
-from ufl.algorithms.multifunction import MultiFunction
+from ufl.corealg.multifunction import MultiFunction
 from ufl.algorithms.transformer import Transformer, is_post_handler, \
-                                       transform, transform_integrands, apply_transformer, \
+                                       apply_transformer, \
                                        ReuseTransformer, ufl2ufl, \
                                        CopyTransformer, ufl2uflcopy, \
                                        VariableStripper, strip_variables
@@ -68,37 +72,19 @@ from ufl.algorithms.expand_indices import expand_indices, purge_list_tensors
 from ufl.algorithms.propagate_restrictions import propagate_restrictions
 
 # Utilities for transforming complete Forms into other Forms
-from ufl.algorithms.formtransformations import compute_form_adjoint, compute_form_action, compute_energy_norm, \
-                                               compute_form_lhs, compute_form_rhs, compute_form_functional, \
-                                               compute_form_arities
+from ufl.algorithms.formtransformations import (
+    compute_form_adjoint, compute_form_action, compute_energy_norm,
+    compute_form_lhs, compute_form_rhs,
+    compute_form_functional, compute_form_arities)
 
 # Utilities for Automatic Functional Differentiation
 from ufl.algorithms.ad import expand_derivatives #, compute_diff, propagate_spatial_derivatives, compute_form_derivative
 
-# Utilities for working with linearized computational graphs
-from ufl.algorithms.graph import Graph, format_graph, rebuild_tree, partition # TODO: Add more imports here
-
-# Utilities for UFL object printing
-from ufl.algorithms.printing import integral_info, form_info, tree_format
-from ufl.algorithms.ufl2latex import ufl2latex, ufl2tex, ufl2pdf, forms2latexdocument
-from ufl.algorithms.ufl2dot import ufl2dot
-
 # Utilities for form file handling
 from ufl.algorithms.formfiles import read_ufl_file, load_ufl_file, load_forms
 
-# State of files (in the opinion of Martin):
-#    traversal.py           - Ok.
-#    analysis.py            - Ok, some unused stuff.
-#    formdata.py            - Probably don't need both self.unique_elements and self.sub_elements?
-#                             Need to improve quadrature order estimation.
-#    graph.py               - Work in progress, works ok so far.
-#    predicates.py          - is_multilinear seems ok but needs testing.
-#    checks.py              - Ok, more checks are welcome.
-#    formfiles.py           - Ok.
-#    transformations.py     - Ok.
-#    formtransformations.py - Ok? Needs testing.
-#    ad.py                  - Ok?
-#    printing.py            - Ok.
-#    latextools.py          - Ok.
-#    ufl2latex.py           - Fix precedence stuff.
-#    ufl2dot.py             - Rework with graph tools.
+
+# Utilities for UFL object printing
+from ufl.formatting.printing import integral_info, form_info, tree_format
+from ufl.formatting.ufl2latex import ufl2latex, ufl2tex, ufl2pdf, forms2latexdocument
+from ufl.formatting.ufl2dot import ufl2dot
