@@ -22,10 +22,11 @@ from ufl.assertions import ufl_assert
 from ufl.log import error
 
 from ufl.core.terminal import Terminal
+from ufl.classes import Coefficient, Argument, GeometricQuantity, Restricted
 from ufl.corealg.multifunction import MultiFunction
 from ufl.corealg.map_dag import map_expr_dag
 from ufl.algorithms.map_integrands import map_integrand_dags
-from ufl.measure import Measure
+from ufl.measure import integral_type_to_measure_name
 
 class RestrictionPropagator(MultiFunction):
     def __init__(self, side=None):
@@ -91,6 +92,17 @@ class RestrictionPropagator(MultiFunction):
         "Strip variable."
         return op
 
+    def reference_value(self, o):
+        "Reference value of something follows same restriction rule as the underlying object."
+        f, = o.ufl_operands
+        assert f._ufl_is_terminal_
+        g = self(f)
+        if isinstance(g, Restricted):
+            side = g.side()
+            return o(side)
+        else:
+            return o
+
     # --- Rules for terminals
 
     # Default: Literals should ignore restriction
@@ -177,6 +189,6 @@ class RestrictionPropagator(MultiFunction):
 
 def apply_restrictions(expression):
     "Propagate restriction nodes to wrap differential terminals directly."
-    integral_types = [Measure.INTERIOR_FACET, Measure.INTERIOR_FACET_HORIZ, Measure.INTERIOR_FACET_VERT]
+    integral_types = [k for k in integral_type_to_measure_name.keys() if k.startswith("interior_facet")]
     rules = RestrictionPropagator()
     return map_integrand_dags(rules, expression, only_integral_type=integral_types)
