@@ -36,11 +36,11 @@ class ArgumentDependencyExtracter(Transformer):
         Transformer.__init__(self)
         self._empty = frozenset()
 
-    def expr(self, o, *opdeps):
-        "Default for nonterminals: nonlinear in all operands."
+    def operator(self, o, *opdeps):
+        "Default for Operators: nonlinear in all operands."
         for d in opdeps:
             if d:
-                raise NotMultiLinearException, repr(o)
+                raise NotMultiLinearException(repr(o))
         return self._empty
 
     def terminal(self, o):
@@ -49,7 +49,7 @@ class ArgumentDependencyExtracter(Transformer):
 
     def variable(self, o):
         # Check variable cache to reuse previously transformed variable if possible
-        e, l = o.operands()
+        e, l = o.ufl_operands
         d = self._variable_cache.get(l)
         if d is None:
             # Visit the expression our variable represents
@@ -64,12 +64,18 @@ class ArgumentDependencyExtracter(Transformer):
     def linear(self, o, a):
         "Nonterminals that are linear with a single argument."
         return a
+    nabla_grad = linear
+    nabla_div = linear
     grad = linear
     div = linear
     curl = linear
+    reference_grad = linear
+    reference_div = linear
+    reference_curl = linear
     transposed = linear
     trace = linear
     skew = linear
+    sym = linear
     positive_restricted = linear
     negative_restricted = linear
     cell_avg = linear
@@ -83,7 +89,7 @@ class ArgumentDependencyExtracter(Transformer):
 
     def variable_derivative(self, o, a, b):
         if b:
-            raise NotMultiLinearException, repr(o)
+            raise NotMultiLinearException(repr(o))
         return a
 
     def component_tensor(self, o, f, i):
@@ -94,19 +100,26 @@ class ArgumentDependencyExtracter(Transformer):
         d = opdeps[0]
         for d2 in opdeps[1:]:
             if not d == d2:
-                raise NotMultiLinearException, repr(o)
+                raise NotMultiLinearException(repr(o))
         return d
 
     def conditional(self, o, cond, t, f):
         "Considering EQ, NE, LE, GE, LT, GT nonlinear in this context."
         if cond or (not t == f):
-            raise NotMultiLinearException, repr(o)
+            raise NotMultiLinearException(repr(o))
         return t
 
+    def min_value(self, o, l, r):
+        "Considering min, max nonlinear in this context."
+        if l or r:
+            raise NotMultiLinearException(repr(o))
+        return t
+    max_value = min_value
+
     def division(self, o, a, b):
         "Arguments cannot be in the denominator."
         if b:
-            raise NotMultiLinearException, repr(o)
+            raise NotMultiLinearException(repr(o))
         return a
 
     def index_sum(self, o, f, i):
@@ -145,7 +158,7 @@ class ArgumentDependencyExtracter(Transformer):
                 cd = (ad | bd) - none
                 # build frozenset cd with the combined Argument dependencies from ad and bd
                 if not len(cd) == len(ad - none) + len(bd - none):
-                    raise NotMultiLinearException, repr(o)
+                    raise NotMultiLinearException(repr(o))
                 # remember this dependency combination
                 if cd:
                     c.append(cd)
