@@ -19,9 +19,6 @@
 #
 # Modified by Anders Logg, 2009-2010
 # Modified by Jan Blechta, 2012
-#
-# First added:  2008-05-07
-# Last changed: 2012-11-04
 
 from ufl.assertions import ufl_assert
 from ufl.log import warning
@@ -48,19 +45,11 @@ class SumDegreeEstimator(Transformer):
             return 0
         else:
             # As a heuristic, just returning domain degree to bump up degree somewhat
-            x = v.domain().coordinates()
-            if x is None:
-                return 1
-            else:
-                return x.element().degree()
+            return v.domain().coordinate_element().degree()
 
     def spatial_coordinate(self, v):
         "A coordinate provides additional degrees depending on coordinate field of domain."
-        x = v.domain().coordinates()
-        if x is None:
-            return 1
-        else:
-            return x.element().degree()
+        return v.domain().coordinate_element().degree()
 
     def cell_coordinate(self, v):
         "A coordinate provides one additional degree."
@@ -75,7 +64,7 @@ class SumDegreeEstimator(Transformer):
         """A form argument provides a degree depending on the element,
         or the default degree if the element has no degree."""
         e = v.element()
-        e = self.element_replace_map.get(e,e)
+        e = self.element_replace_map.get(e, e)
         d = e.degree() # FIXME: Use component to improve accuracy for mixed elements
         if d is None:
             d = self.default_degree
@@ -90,6 +79,7 @@ class SumDegreeEstimator(Transformer):
         else:
             # if tuple, do not reduce
             return f
+
     def _add_degrees(self, v, *ops):
         if all(isinstance(o, int) for o in ops):
             return sum(ops)
@@ -99,18 +89,20 @@ class SumDegreeEstimator(Transformer):
             # 0 to (0, 0) when making tempops]
             tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
             return tuple(map(sum, zip(*tempops)))
+
     def _max_degrees(self, v, *ops):
         if all(isinstance(o, int) for o in ops):
             return max(ops + (0,))
         else:
             tempops = [foo if isinstance(foo, tuple) else (foo, foo) for foo in ops]
             return tuple(map(max, zip(*tempops)))
+
     def _not_handled(self, v, *args):
-        error("Missing degree handler for type %s" % v._uflclass.__name__)
+        error("Missing degree handler for type %s" % v._ufl_class_.__name__)
 
     def expr(self, v, *ops):
         "For most operators we take the max degree of its operands."
-        warning("Missing degree estimation handler for type %s" % v._uflclass.__name__)
+        warning("Missing degree estimation handler for type %s" % v._ufl_class_.__name__)
         return self._add_degrees(v, *ops)
 
     # Utility types with no degree concept
@@ -192,7 +184,7 @@ class SumDegreeEstimator(Transformer):
         degree(a**b) == degree(a)*b
         otherwise use the heuristic
         degree(a**b) == degree(a)*2"""
-        f, g = v.operands()
+        f, g = v.ufl_operands
         try:
             gi = abs(int(g))
             if isinstance(a, int):
@@ -216,9 +208,9 @@ class SumDegreeEstimator(Transformer):
         """
         #print "estimate",a,b
         if a or b:
-            return max(a,b)+2
+            return self._add_degrees(v, self._max_degrees(v, a, b), 2)
         else:
-            return max(a,b)
+            return self._max_degrees(v, a, b)
 
 
     def math_function(self, v, a):
@@ -229,7 +221,7 @@ class SumDegreeEstimator(Transformer):
         gives a somewhat high integration degree.
         """
         if a:
-            return a+2
+            return self._add_degrees(v, a, 2)
         else:
             return a
 
@@ -241,7 +233,7 @@ class SumDegreeEstimator(Transformer):
         gives a somewhat high integration degree.
         """
         if x:
-            return x+2
+            return self._add_degrees(v, x, 2)
         else:
             return x
 
@@ -258,7 +250,13 @@ class SumDegreeEstimator(Transformer):
         For improving accuracy of quadrature
         near condition transition surface
         quadrature order must be adjusted manually."""
-        return max(t, f)
+        return self._max_degrees(v, t, f)
+
+    def min_value(self, v, l, r):
+        """Same as conditional."""
+        return self._max_degrees(v, l, r)
+    max_value = min_value
+
 
 def estimate_total_polynomial_degree(e, default_degree=1, element_replace_map={}):
     """Estimate total polynomial degree of integrand.
@@ -279,4 +277,33 @@ def estimate_total_polynomial_degree(e, default_degree=1, element_replace_map={}
         degrees = [de.visit(e.integrand())]
     else:
         degrees = [de.visit(e)]
-    return max(degrees + [0])
+    degree = max(degrees) if degrees else default_degree
+    return degree
+
+
+# TODO: Do these contain useful ideas or should we just delete them?
+
+
+def __unused__extract_max_quadrature_element_degree(integral):
+    """Extract quadrature integration order from quadrature
+    elements in integral. Returns None if not found."""
+    quadrature_elements = [e for e in extract_elements(integral) if "Quadrature" in e.family()]
+    degrees = [element.degree() for element in quadrature_elements]
+    degrees = [q for q in degrees if not q is None]
+    if not degrees:
+        return None
+    max_degree = quadrature_elements[0].degree()
+    ufl_assert(all(max_degree == q for q in degrees),
+               "Incompatible quadrature elements specified (orders must be equal).")
+    return max_degree
+
+
+def __unused__estimate_quadrature_degree(integral):
+    "Estimate the necessary quadrature order for integral using the sum of argument degrees."
+    arguments = extract_arguments(integral)
+    degrees = [v.element().degree() for v in arguments]
+    if len(arguments) == 0:
+        return None
+    if len(arguments) == 1:
+        return 2*degrees[0]
+    return sum(degrees)
