@@ -1,7 +1,5 @@
 """A collection of utility algorithms for handling UFL files."""
 
-from __future__ import with_statement
-
 # Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
@@ -21,17 +19,14 @@ from __future__ import with_statement
 #
 # Modified by Anders Logg, 2008-2009.
 # Modified by Marie E. Rognes, 2011.
-#
-# First added:  2008-03-14
-# Last changed: 2012-12-11
 
 import os, re
 from ufl.log import error, warning
-from ufl.common import sorted_items
+from ufl.utils.sorting import sorted_by_key
 from ufl.assertions import ufl_assert
 from ufl.form import Form
 from ufl.finiteelement import FiniteElementBase
-from ufl.expr import Expr
+from ufl.core.expr import Expr
 from ufl.argument import Argument
 from ufl.coefficient import Coefficient
 
@@ -45,9 +40,10 @@ class FileData(object):
         self.object_by_name = {}
         self.reserved_objects = {}
 
-    def __nonzero__(self):
+    def __bool__(self):
         return bool(self.elements or self.coefficients or self.forms or self.expressions or\
                     self.object_names or self.object_by_name or self.reserved_objects)
+    __nonzero__ = __bool__
 
 infostring = """An exception occured during evaluation of form file.
 To help you find the location of the error, a temporary script
@@ -87,14 +83,14 @@ def execute_ufl_code(uflcode, filename):
     namespace = {}
     try:
         pycode = "from ufl import *\n" + uflcode
-        exec pycode in namespace
+        exec(pycode, namespace)
     except:
         # Dump python code for debugging if this fails
         basename = os.path.splitext(os.path.basename(filename))[0]
         basename = "%s_debug" % basename
         pyname = "%s.py" % basename
         pycode = "#!/usr/bin/env python\nfrom ufl import *\nset_level(DEBUG)\n" + uflcode
-        with file(pyname, "w") as f:
+        with open(pyname, "w") as f:
             f.write(pycode)
         warning(infostring % pyname)
         m = __import__(basename)
@@ -110,7 +106,7 @@ def interpret_ufl_namespace(namespace):
     # The use of id(obj) as key in object_names is necessary
     # because we need to distinguish between instances,
     # and not just between objects with different values.
-    for name, value in sorted_items(namespace):
+    for name, value in sorted_by_key(namespace):
         # Store objects by reserved name OR instance id
         reserved_names = ("unknown",) # Currently only one reserved name
         if name in reserved_names:
