@@ -1,6 +1,6 @@
 """Forward mode AD implementation."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,25 +21,22 @@
 # Modified by Garth N. Wells, 2010.
 # Modified by Kristian B. Oelgaard, 2011
 # Modified by Jan Blechta, 2012.
-#
-# First added:  2008-08-19
-# Last changed: 2013-08-12
 
-from itertools import izip
+from six.moves import xrange as range
+from six.moves import zip
+
 from math import pi
 from ufl.log import error, warning, debug
 from ufl.assertions import ufl_assert
 from ufl.common import unzip, subdict, lstr
-from ufl.indexutils import unique_indices
 
 # All classes:
-from ufl.terminal import Terminal
-from ufl.operatorbase import Tuple
+from ufl.core.terminal import Terminal
 from ufl.constantvalue import ConstantValue, Zero, IntValue, Identity,\
     is_true_ufl_scalar, is_ufl_scalar
 from ufl.variable import Variable
-from ufl.coefficient import ConstantBase, Coefficient, FormArgument
-from ufl.indexing import MultiIndex, Index, FixedIndex, indices
+from ufl.coefficient import Coefficient, FormArgument
+from ufl.core.multiindex import MultiIndex, Index, FixedIndex, indices
 from ufl.indexed import Indexed
 from ufl.indexsum import IndexSum
 from ufl.tensors import ListTensor, ComponentTensor, as_tensor, as_scalar, unit_indexed_tensor, unwrap_list_tensor
@@ -51,7 +48,7 @@ from ufl.restriction import Restricted, PositiveRestricted, NegativeRestricted
 from ufl.differentiation import Derivative, CoefficientDerivative,\
     VariableDerivative, Grad
 from ufl.conditional import EQ, NE, LE, GE, LT, GT, Conditional
-
+from ufl.exprcontainers import ExprList, ExprMapping
 from ufl.operators import dot, inner, outer, lt, eq, conditional, sign, \
     sqrt, exp, ln, cos, sin, tan, cosh, sinh, tanh, acos, asin, atan, atan_2, \
     erf, bessel_J, bessel_Y, bessel_I, bessel_K, \
@@ -60,61 +57,10 @@ from ufl.algorithms.transformer import Transformer
 
 
 class ForwardAD(Transformer):
-    def __init__(self, spatial_dim, var_shape, var_free_indices, var_index_dimensions, cache=None):
+    def __init__(self, var_shape, cache=None):
         Transformer.__init__(self)
-        ufl_assert(all(isinstance(i, Index) for i in var_free_indices), \
-            "Expecting Index objects.")
-        ufl_assert(all(isinstance(i, Index) for i in var_index_dimensions.keys()), \
-            "Expecting Index objects.")
-        self._spatial_dim = spatial_dim
         self._var_shape = var_shape
-        self._var_free_indices = var_free_indices
-        self._var_index_dimensions = dict(var_index_dimensions)
-        #if self._var_free_indices:
-        #    warning("TODO: Free indices in differentiation variable may be buggy!")
-        self._cache = {} if cache is None else cache
-
-    def _cache_visit(self, o):
-        "Cache hook, disable this by renaming to something else than 'visit'."
-        #debug("Visiting object of type %s." % type(o).__name__)
-
-        # TODO: This doesn't work, why?
-
-        # NB! Cache added in after copying from Transformer
-        c = self._cache.get(o)
-        if c is not None:
-            return c
-
-        # Reuse default visit function
-        r = Transformer.visit(self, o)
-
-        if (c is not None):
-            if r[0].free_indices() != c[0].free_indices():
-                print "="*70
-                print "=== f: Difference between cache and recomputed indices:"
-                print str(c[0].free_indices())
-                print str(r[0].free_indices())
-                print "="*70
-            if r[1].free_indices() != c[1].free_indices():
-                print "="*70
-                print "=== df: Difference between cache and recomputed indices:"
-                print str(c[1].free_indices())
-                print str(r[1].free_indices())
-                print "="*70
-            if (r != c):
-                print "="*70
-                print "=== Difference between cache and recomputed:"
-                print str(c[0])
-                print str(c[1])
-                print "-"*40
-                print str(r[0])
-                print str(r[1])
-                print "="*70
-
-        # NB! Cache added in after copying from Transformer
-        self._cache[o] = r
-
-        return r
+        self._variable_cache = {} if cache is None else cache
 
     def _debug_visit(self, o):
         "Debugging hook, enable this by renaming to 'visit'."
@@ -125,77 +71,20 @@ class ForwardAD(Transformer):
             debug("  o:  %s" % str(o))
             debug("  f:  %s" % str(f))
             debug("  df: %s" % str(df))
-        fi_diff = set(f.free_indices()) ^ set(df.free_indices())
+        fi_diff = set(f.ufl_free_indices) ^ set(df.ufl_free_indices)
         if fi_diff:
             debug("In ForwardAD.visit, got free indices diff:")
             debug("  o:  %s" % str(o))
             debug("  f:  %s" % str(f))
             debug("  df: %s" % str(df))
-            debug("  f.fi():  %s" % lstr(f.free_indices()))
-            debug("  df.fi(): %s" % lstr(df.free_indices()))
+            debug("  f.fi():  %s" % lstr(f.ufl_free_indices))
+            debug("  df.fi(): %s" % lstr(df.ufl_free_indices))
             debug("  fi_diff: %s" % str(fi_diff))
         return r
 
     def _make_zero_diff(self, o):
-        # Define a zero with the right indices
-        # (kind of cumbersome this... any simpler way?)
-        sh = o.shape() + self._var_shape
-        fi = o.free_indices()
-        idims = dict(o.index_dimensions())
-        if self._var_free_indices:
-            # Currently assuming only one free variable index
-            i, = self._var_free_indices
-            if i not in idims:
-                fi = unique_indices(fi + (i,))
-                idims[i] = self._var_index_dimensions[i]
-        fp = Zero(sh, fi, idims)
-        return fp
-
-    def _make_ones_diff(self, o):
-        ufl_assert(o.shape() == self._var_shape, "This is only used by VariableDerivative, yes?")
-        # Define a scalar value with the right indices
-        # (kind of cumbersome this... any simpler way?)
-
-        sh = o.shape()
-        fi = o.free_indices()
-        idims = dict(o.index_dimensions())
-
-        if self._var_free_indices:
-            # Currently assuming only one free variable index
-            i, = self._var_free_indices
-            if i not in idims:
-                fi = unique_indices(fi + (i,))
-                idims[i] = self._var_index_dimensions[i]
-
-        # Create a 1 with index annotations
-        one = IntValue(1, (), fi, idims)
-
-        res = None
-        if sh == ():
-            return one
-        elif len(sh) == 1:
-            # FIXME: If sh == (1,), I think this will get the wrong shape?
-            # I think we should make sure sh=(1,...,1) is always converted to () early.
-            fp = Identity(sh[0])
-        else:
-            ind1 = ()
-            ind2 = ()
-            for d in sh:
-                i, j = indices(2)
-                dij = Identity(d)[i, j]
-                if res is None:
-                    res = dij
-                else:
-                    res *= dij
-                ind1 += (i,)
-                ind2 += (j,)
-            fp = as_tensor(res, ind1 + ind2)
-
-        # Apply index annotations
-        if fi:
-            fp *= one
-
-        return fp
+        # Define a zero with the right shape and indices
+        return Zero(o.ufl_shape + self._var_shape, o.ufl_free_indices, o.ufl_index_dimensions)
 
     # --- Default rules
 
@@ -214,7 +103,7 @@ class ForwardAD(Transformer):
         """Variable objects are just 'labels', so by default the derivative
         of a variable is the derivative of its referenced expression."""
         # Check variable cache to reuse previously transformed variable if possible
-        e, l = o.operands()
+        e, l = o.ufl_operands
         r = self._variable_cache.get(l) # cache contains (v, vp) tuple
         if r is not None:
             return r
@@ -236,7 +125,7 @@ class ForwardAD(Transformer):
         return (o, None) # oprime here should never be used, this might even not be called?
 
     def indexed(self, o):
-        A, jj = o.operands()
+        A, jj = o.ufl_operands
         A2, Ap = self.visit(A)
         o = self.reuse_if_possible(o, A2, jj)
 
@@ -246,7 +135,7 @@ class ForwardAD(Transformer):
             r = Ap.rank() - len(jj)
             if r:
                 ii = indices(r)
-                op = Indexed(Ap, jj._indices + ii)
+                op = Indexed(Ap, MultiIndex(jj.indices() + ii))
                 op = as_tensor(op, ii)
             else:
                 op = Indexed(Ap, jj)
@@ -259,7 +148,7 @@ class ForwardAD(Transformer):
         return (o, op)
 
     def component_tensor(self, o):
-        A, ii = o.operands()
+        A, ii = o.ufl_operands
         A, Ap = self.visit(A)
         o = self.reuse_if_possible(o, A, ii)
 
@@ -267,39 +156,13 @@ class ForwardAD(Transformer):
             op = self._make_zero_diff(o)
         else:
             Ap, jj = as_scalar(Ap)
-            op = ComponentTensor(Ap, ii._indices + jj)
+            op = as_tensor(Ap, ii.indices() + jj)
         return (o, op)
 
     # --- Algebra operators
 
     def index_sum(self, o):
-        A, i = o.operands()
-
-        # Consider the following case:
-        #   (v[i]*u[i]).dx(i)
-        # which is represented like
-        #   SpatialDerivative(IndexSum(..., i), i)
-        # if we move the derivative inside the sum,
-        # then the derivative suddenly gets accumulated,
-        # which is completely wrong!
-        if self._var_free_indices:
-            if i[0] in self._var_free_indices:
-                error("Index scope collision. Work around this by reusing indices less in different expressions.\n"\
-                      "An example where this occurs is (v[i]*v[i]).dx(i) where the same index i\n"\
-                      "is bound to an index sum inside the derivative .dx(i).")
-
-                # TODO: OR get around this by temporarily setting _var_free_indices to nothing?
-                #store = self._var_free_indices
-                #self._var_free_indices = nothing?
-                #visit children
-                #self._var_free_indices = store
-                # But... What would this mean? Will it be correct?
-
-                # TODO: Get around this with relabeling algorithm!
-                # j = Index()
-                # A = relabel(A, { i0: j })
-                # i = (j,)
-
+        A, i = o.ufl_operands
         A2, Ap = self.visit(A)
         o = self.reuse_if_possible(o, A2, i)
         op = IndexSum(Ap, i)
@@ -317,7 +180,7 @@ class ForwardAD(Transformer):
         # Get operands and their derivatives
         ops2, dops2 = unzip(ops)
         o = self.reuse_if_possible(o, *ops2)
-        for i in xrange(len(ops)):
+        for i in range(len(ops)):
             # Get scalar representation of differentiated value of operand i
             dop = dops2[i]
             dop, ii = as_scalar(dop)
@@ -360,10 +223,10 @@ class ForwardAD(Transformer):
 
         # Debugging prints, should never happen:
         if not is_true_ufl_scalar(f):
-            print ":"*80
-            print "f =", str(f)
-            print "g =", str(g)
-            print ":"*80
+            print(":"*80)
+            print("f =", str(f))
+            print("g =", str(g))
+            print(":"*80)
         ufl_assert(is_true_ufl_scalar(f), "Expecting scalar expression f in f**g.")
         ufl_assert(is_true_ufl_scalar(g), "Expecting scalar expression g in f**g.")
 
@@ -503,7 +366,7 @@ class ForwardAD(Transformer):
             error("Differentiation of bessel function w.r.t. nu is not supported.")
         f, fp = x
         o = self.reuse_if_possible(o, nu, f)
-        if nu == 0:
+        if isinstance(nu, Zero):
             op = -bessel_J(1, f)
         else:
             op = 0.5 * (bessel_J(nu-1, f) - bessel_J(nu+1, f))
@@ -515,7 +378,7 @@ class ForwardAD(Transformer):
             error("Differentiation of bessel function w.r.t. nu is not supported.")
         f, fp = x
         o = self.reuse_if_possible(o, nu, f)
-        if nu == 0:
+        if isinstance(nu, Zero):
             op = -bessel_Y(1, f)
         else:
             op = 0.5 * (bessel_Y(nu-1, f) - bessel_Y(nu+1, f))
@@ -527,7 +390,7 @@ class ForwardAD(Transformer):
             error("Differentiation of bessel function w.r.t. nu is not supported.")
         f, fp = x
         o = self.reuse_if_possible(o, nu, f)
-        if nu == 0:
+        if isinstance(nu, Zero):
             op = bessel_I(1, f)
         else:
             op = 0.5 * (bessel_I(nu-1, f) + bessel_I(nu+1, f))
@@ -539,7 +402,7 @@ class ForwardAD(Transformer):
             error("Differentiation of bessel function w.r.t. nu is not supported.")
         f, fp = x
         o = self.reuse_if_possible(o, nu, f)
-        if nu == 0:
+        if isinstance(nu, Zero):
             op = -bessel_K(1, f)
         else:
             op = -0.5 * (bessel_K(nu-1, f) + bessel_K(nu+1, f))
@@ -599,14 +462,27 @@ class ForwardAD(Transformer):
     def conditional(self, o, c, t, f):
         o = self.reuse_if_possible(o, c[0], t[0], f[0])
         if isinstance(t[1], Zero) and isinstance(f[1], Zero):
-            tp = t[1] # Assuming t[1] and f[1] have the same indices here, which should be the case
-            fi = tp.free_indices()
-            fid = subdict(tp.index_dimensions(), fi)
-            op = Zero(tp.shape(), fi, fid)
+            # Assuming t[1] and f[1] have the same indices here, which should be the case
+            op = t[1]
         else:
+            # Placing t[1],f[1] outside here to avoid getting arguments inside conditionals
             op = conditional(c[0], 1, 0)*t[1] + conditional(c[0], 0, 1)*f[1]
         return (o, op)
 
+    def max_value(self, o, x, y):
+        #d/dx max(f, g) =
+        # f > g: df/dx
+        # f < g: dg/dx
+        op = conditional(x[0] > y[0], x[1], y[1])
+        return (o, op)
+
+    def min_value(self, o, x, y):
+        #d/dx min(f, g) =
+        # f < g: df/dx
+        # else: dg/dx
+        op = conditional(x[0] < y[0], x[1], y[1])
+        return (o, op)
+
     # --- Other derivatives
 
     def derivative(self, o):
@@ -615,49 +491,65 @@ class ForwardAD(Transformer):
     def grad(self, o):
         error("FIXME")
 
-    def xspatial_derivative(self, o): # FIXME: Translate to grad situation
-        # If we hit this type, it has already been propagated
-        # to a terminal, so we can simply apply our derivative
-        # to its operand since differentiation commutes.
-        f, ii = o.operands()
-        f, fp = self.visit(f)
-        o = self.reuse_if_possible(o, f, ii)
-
-        # FIXME: Make plenty of test cases around this kind of situation to document what's going on...
-        if fp.is_cellwise_constant():
-            sh = fp.shape()
-            fi = fp.free_indices()
-            idims = dict(fp.index_dimensions())
-            j, = ii
-            if isinstance(j, Index) and j not in idims:
-                fi = fi + (j,)
-                idims.update(ii.index_dimensions())
-            oprime = Zero(sh, fi, idims)
-            #oprime = self._make_zero_diff(o) # FIXME: Can we just use this?
-        else:
-            oprime = SpatialDerivative(fp, ii)
-        return (o, oprime)
-
+# TODO: Add a ReferenceGradAD ruleset
 class GradAD(ForwardAD):
-    def __init__(self, spatial_dim, cache=None):
-        ForwardAD.__init__(self, spatial_dim,
-                           var_shape=(spatial_dim,),
-                           var_free_indices=(),
-                           var_index_dimensions={},
-                           cache=cache)
-        self._Id = Identity(spatial_dim)
+    def __init__(self, geometric_dimension, cache=None):
+        ForwardAD.__init__(self, var_shape=(geometric_dimension,), cache=cache)
+
+    def geometric_quantity(self, o):
+        "Represent grad(g) as Grad(g)."
+        # Collapse gradient of cellwise constant function to zero
+        if o.is_cellwise_constant():
+            return self.terminal(o)
+        return (o, Grad(o))
 
     def spatial_coordinate(self, o):
         "Gradient of x w.r.t. x is Id."
+        if not hasattr(self, '_Id'):
+            gdim = o.geometric_dimension()
+            self._Id = Identity(gdim)
         return (o, self._Id)
 
-    # This is implicit for all terminals, but just to make this clear to the reader:
-    facet_normal = ForwardAD.terminal # returns zero
-    constant = ForwardAD.terminal # returns zero
+    def cell_coordinate(self, o):
+        "Gradient of X w.r.t. x is K. But I'm not sure if we want to allow this."
+        error("This has not been validated. Does it make sense to do this here?")
+        K = JacobianInverse(o.domain())
+        return (o, K)
 
-    def form_argument(self, o):
+    # TODO: Implement rules for some of these types?
+
+    def facet_coordinate(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def jacobian(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def jacobian_determinant(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def jacobian_inverse(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def facet_jacobian(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def facet_jacobian_determinant(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def facet_jacobian_inverse(self, o):
+        error("Not expecting this low level type in AD.")
+
+    def argument(self, o):
         "Represent grad(f) as Grad(f)."
-        # Collapse gradient of cellwise function to zero
+        # Collapse gradient of cellwise constant function to zero
+        # FIXME: Enable this after fixing issue#13
+        #if o.is_cellwise_constant():
+        #    return zero(...) # TODO: zero annotated with argument
+        return (o, Grad(o))
+
+    def coefficient(self, o):
+        "Represent grad(f) as Grad(f)."
+        # Collapse gradient of cellwise constant function to zero
         if o.is_cellwise_constant():
             return self.terminal(o)
         return (o, Grad(o))
@@ -671,18 +563,18 @@ class GradAD(ForwardAD):
         #    return self.terminal(o)
         # TODO: Maybe we can ask "f.has_derivatives_of_order(n)" to check
         #       if we should make a zero here?
+        # 1) n = count number of Grads, get f
+        # 2) if not f.has_derivatives(n): return zero(...)
 
-        f, = o.operands()
-        ufl_assert(isinstance(f, (Grad,Terminal)),
+        f, = o.ufl_operands
+        ufl_assert(isinstance(f, (Grad, Terminal)),
                    "Expecting derivatives of child to be already expanded.")
         return (o, Grad(o))
 
 class VariableAD(ForwardAD):
-    def __init__(self, spatial_dim, var, cache=None):
-        ForwardAD.__init__(self, spatial_dim, var_shape=var.shape(),
-                           var_free_indices=var.free_indices(),
-                           var_index_dimensions=var.index_dimensions(),
-                           cache=cache)
+    def __init__(self, var, cache=None):
+        ForwardAD.__init__(self, var_shape=var.ufl_shape, cache=cache)
+        ufl_assert(not var.ufl_free_indices, "Differentiation variable cannot have free indices.")
         self._variable = var
 
     def grad(self, o):
@@ -690,17 +582,43 @@ class VariableAD(ForwardAD):
         # to a coefficient, so it cannot depend on the variable. # FIXME: Assert this!
         return self.terminal(o)
 
+    def _make_self_diff_identity(self, o):
+        sh = o.ufl_shape
+        res = None
+        if sh == ():
+            # Scalar dv/dv is scalar
+            return IntValue(1)
+        elif len(sh) == 1:
+            # Vector v makes dv/dv the identity matrix
+            return Identity(sh[0])
+        else:
+            # Tensor v makes dv/dv some kind of higher rank identity tensor
+            ind1 = ()
+            ind2 = ()
+            for d in sh:
+                i, j = indices(2)
+                dij = Identity(d)[i, j]
+                if res is None:
+                    res = dij
+                else:
+                    res *= dij
+                ind1 += (i,)
+                ind2 += (j,)
+            fp = as_tensor(res, ind1 + ind2)
+
+        return fp
+
     def variable(self, o):
         # Check cache
-        e, l = o.operands()
+        e, l = o.ufl_operands
         c = self._variable_cache.get(l)
 
         if c is not None:
             return c
 
         if o.label() == self._variable.label():
-            # dv/dv = "1"
-            op = self._make_ones_diff(o)
+            # dv/dv = identity of rank 2*rank(v)
+            op = self._make_self_diff_identity(o)
         else:
             # differentiate expression behind variable
             e2, ep = self.visit(e)
@@ -714,14 +632,15 @@ class VariableAD(ForwardAD):
 
 class CoefficientAD(ForwardAD):
     "Apply AFD (Automatic Functional Differentiation) to expression."
-    def __init__(self, spatial_dim, coefficients, arguments, coefficient_derivatives, cache=None):
-        ForwardAD.__init__(self, spatial_dim, var_shape=(), var_free_indices=(),
-                           var_index_dimensions={}, cache=cache)
+    def __init__(self, coefficients, arguments, coefficient_derivatives, cache=None):
+        ForwardAD.__init__(self, var_shape=(), cache=cache)
+        ufl_assert(isinstance(coefficients, ExprList), "Expecting a ExprList.")
+        ufl_assert(isinstance(arguments, ExprList), "Expecting a ExprList.")
+        ufl_assert(isinstance(coefficient_derivatives, ExprMapping), "Expecting a ExprList.")
         self._v = arguments
         self._w = coefficients
-        self._cd = coefficient_derivatives
-        ufl_assert(isinstance(self._w, Tuple), "Expecting a Tuple.")
-        ufl_assert(isinstance(self._v, Tuple), "Expecting a Tuple.")
+        cd = coefficient_derivatives.ufl_operands
+        self._cd = dict((cd[2*i], cd[2*i+1]) for i in range(len(cd)//2))
 
     def coefficient(self, o):
         # Define dw/dw := d/ds [w + s v] = v
@@ -732,15 +651,15 @@ class CoefficientAD(ForwardAD):
         debug("self._v = %s" % self._v)
 
         # Find o among w
-        for (w, v) in izip(self._w, self._v):
+        for (w, v) in zip(self._w, self._v):
             if o == w:
                 return (w, v)
 
         # If o is not among coefficient derivatives, return do/dw=0
-        oprimesum = Zero(o.shape())
-        oprimes = self._cd._data.get(o)
+        oprimesum = Zero(o.ufl_shape)
+        oprimes = self._cd.get(o)
         if oprimes is None:
-            if self._cd._data:
+            if self._cd:
                 # TODO: Make it possible to silence this message in particular?
                 #       It may be good to have for debugging...
                 warning("Assuming d{%s}/d{%s} = 0." % (o, self._w))
@@ -755,9 +674,9 @@ class CoefficientAD(ForwardAD):
             # Since we may actually have a tuple of oprimes and vs in a
             # 'mixed' space, sum over them all to get the complete inner
             # product. Using indices to define a non-compound inner product.
-            for (oprime, v) in izip(oprimes, self._v):
+            for (oprime, v) in zip(oprimes, self._v):
                 so, oi = as_scalar(oprime)
-                rv = len(v.shape())
+                rv = len(v.ufl_shape)
                 oi1 = oi[:-rv]
                 oi2 = oi[-rv:]
                 prod = so*v[oi2]
@@ -783,31 +702,31 @@ class CoefficientAD(ForwardAD):
         ngrads = 0
         o = g
         while isinstance(o, Grad):
-            o, = o.operands()
+            o, = o.ufl_operands
             ngrads += 1
         if not isinstance(o, FormArgument):
             error("Expecting gradient of a FormArgument, not %r" % (o,))
 
         def apply_grads(f):
             if not isinstance(f, FormArgument):
-                print ','*60
-                print f
-                print o
-                print g
-                print ','*60
+                print(','*60)
+                print(f)
+                print(o)
+                print(g)
+                print(','*60)
                 error("What?")
             for i in range(ngrads):
                 f = Grad(f)
             return f
 
         # Find o among all w without any indexing, which makes this easy
-        for (w, v) in izip(self._w, self._v):
+        for (w, v) in zip(self._w, self._v):
             if o == w and isinstance(v, FormArgument):
                 # Case: d/dt [w + t v]
                 return (g, apply_grads(v))
 
         # If o is not among coefficient derivatives, return do/dw=0
-        gprimesum = Zero(g.shape())
+        gprimesum = Zero(g.ufl_shape)
 
         def analyse_variation_argument(v):
             # Analyse variation argument
@@ -817,7 +736,7 @@ class CoefficientAD(ForwardAD):
             elif isinstance(v, Indexed):
                 # Case: d/dt [w + t v[...]]
                 # Case: d/dt [w[...] + t v[...]]
-                vval, vcomp = v.operands()
+                vval, vcomp = v.ufl_operands
                 vcomp = tuple(vcomp)
             else:
                 error("Expecting argument or component of argument.")
@@ -839,12 +758,12 @@ class CoefficientAD(ForwardAD):
             return gprimeterm
 
         # Accumulate contributions from variations in different components
-        for (w, v) in izip(self._w, self._v):
+        for (w, v) in zip(self._w, self._v):
 
             # Analyse differentiation variable coefficient
             if isinstance(w, FormArgument):
                 if not w == o: continue
-                wshape = w.shape()
+                wshape = w.ufl_shape
 
                 if isinstance(v, FormArgument):
                     # Case: d/dt [w + t v]
@@ -868,12 +787,12 @@ class CoefficientAD(ForwardAD):
             elif isinstance(w, Indexed): # This path is tested in unit tests, but not actually used?
                 # Case: d/dt [w[...] + t v[...]]
                 # Case: d/dt [w[...] + t v]
-                wval, wcomp = w.operands()
+                wval, wcomp = w.ufl_operands
                 if not wval == o: continue
                 assert isinstance(wval, FormArgument)
                 ufl_assert(all(isinstance(k, FixedIndex) for k in wcomp),
                            "Expecting only fixed indices in differentiation variable.")
-                wshape = wval.shape()
+                wshape = wval.ufl_shape
 
                 vval, vcomp = analyse_variation_argument(v)
                 gprimesum = gprimesum + compute_gprimeterm(ngrads, vval, vcomp, wshape, wcomp)
@@ -881,12 +800,12 @@ class CoefficientAD(ForwardAD):
             else:
                 error("Expecting coefficient or component of coefficient.")
 
-        # FIXME: Handle other coefficient derivatives: oprimes = self._cd._data.get(o)
+        # FIXME: Handle other coefficient derivatives: oprimes = self._cd.get(o)
 
         if 0:
-            oprimes = self._cd._data.get(o)
+            oprimes = self._cd.get(o)
             if oprimes is None:
-                if self._cd._data:
+                if self._cd:
                     # TODO: Make it possible to silence this message in particular?
                     #       It may be good to have for debugging...
                     warning("Assuming d{%s}/d{%s} = 0." % (o, self._w))
@@ -901,10 +820,10 @@ class CoefficientAD(ForwardAD):
                 # Since we may actually have a tuple of oprimes and vs in a
                 # 'mixed' space, sum over them all to get the complete inner
                 # product. Using indices to define a non-compound inner product.
-                for (oprime, v) in izip(oprimes, self._v):
+                for (oprime, v) in zip(oprimes, self._v):
                     error("FIXME: Figure out how to do this with ngrads")
                     so, oi = as_scalar(oprime)
-                    rv = len(v.shape())
+                    rv = len(v.ufl_shape)
                     oi1 = oi[:-rv]
                     oi2 = oi[-rv:]
                     prod = so*v[oi2]
@@ -917,7 +836,7 @@ class CoefficientAD(ForwardAD):
 
     def variable(self, o):
         # Check variable cache to reuse previously transformed variable if possible
-        e, l = o.operands()
+        e, l = o.ufl_operands
         c = self._variable_cache.get(l)
         if c is not None:
             return c
@@ -934,29 +853,29 @@ class CoefficientAD(ForwardAD):
         self._variable_cache[l] = c
         return c
 
-def compute_grad_forward_ad(f, dim):
-    alg = GradAD(dim)
+def compute_grad_forward_ad(f, geometric_dimension):
+    alg = GradAD(geometric_dimension)
     e, ediff = alg.visit(f)
     return ediff
 
-def compute_variable_forward_ad(f, v, dim):
-    alg = VariableAD(dim, v)
+def compute_variable_forward_ad(f, v):
+    alg = VariableAD(v)
     e, ediff = alg.visit(f)
     return ediff
 
-def compute_coefficient_forward_ad(f, w, v, cd, dim):
-    alg = CoefficientAD(dim, w, v, cd)
+def compute_coefficient_forward_ad(f, w, v, cd):
+    alg = CoefficientAD(w, v, cd)
     e, ediff = alg.visit(f)
     return ediff
 
-def apply_nested_forward_ad(expr, dim):
-    if isinstance(expr, Terminal):
+def apply_nested_forward_ad(expr):
+    if expr._ufl_is_terminal_:
         # A terminal needs no differentiation applied
         return expr
     elif not isinstance(expr, Derivative):
         # Apply AD recursively to children
-        preops = expr.operands()
-        postops = tuple(apply_nested_forward_ad(o, dim) for o in preops)
+        preops = expr.ufl_operands
+        postops = tuple(apply_nested_forward_ad(o) for o in preops)
         # Reconstruct if necessary
         need_reconstruct = not (preops == postops) # FIXME: Is this efficient? O(n)?
         if need_reconstruct:
@@ -964,22 +883,23 @@ def apply_nested_forward_ad(expr, dim):
         return expr
     elif isinstance(expr, Grad):
         # Apply AD recursively to children
-        f, = expr.operands()
-        f = apply_nested_forward_ad(f, dim)
+        f, = expr.ufl_operands
+        f = apply_nested_forward_ad(f)
         # Apply Grad-specialized AD to expanded child
-        return compute_grad_forward_ad(f, dim)
+        gdim = expr.ufl_shape[-1]
+        return compute_grad_forward_ad(f, gdim)
     elif isinstance(expr, VariableDerivative):
         # Apply AD recursively to children
-        f, v = expr.operands()
-        f = apply_nested_forward_ad(f, dim)
+        f, v = expr.ufl_operands
+        f = apply_nested_forward_ad(f)
         # Apply Variable-specialized AD to expanded child
-        return compute_variable_forward_ad(f, v, dim)
+        return compute_variable_forward_ad(f, v)
     elif isinstance(expr, CoefficientDerivative):
         # Apply AD recursively to children
-        f, w, v, cd = expr.operands()
-        f = apply_nested_forward_ad(f, dim)
+        f, w, v, cd = expr.ufl_operands
+        f = apply_nested_forward_ad(f)
         # Apply Coefficient-specialized AD to expanded child
-        return compute_coefficient_forward_ad(f, w, v, cd, dim)
+        return compute_coefficient_forward_ad(f, w, v, cd)
     else:
         error("Unknown type.")
 
@@ -1005,6 +925,7 @@ class UnusedADRules(object):
     def inner(self, o, a, b):
         a, ap = a
         b, bp = b
+        # FIXME: Rewrite with index notation (if necessary, depends on shapes)
         # NB! Using b : ap because derivative axis should be
         # last, in case of nonscalar differentiation variable!
         return (o, inner(b, ap) + inner(a, bp)) # FIXME: Not correct, inner requires equal shapes!
@@ -1012,6 +933,7 @@ class UnusedADRules(object):
     def dot(self, o, a, b):
         a, ap = a
         b, bp = b
+        # FIXME: Rewrite with index notation (if necessary, depends on shapes)
         # NB! Using b . ap because derivative axis should be
         # last, in case of nonscalar differentiation variable!
         return (o, dot(b, ap) + dot(a, bp))
@@ -1033,11 +955,10 @@ class UnusedADRules(object):
     curl = commute
     def grad(self, o, a):
         a, aprime = a
-        cell = aprime.cell()
-        if cell is None: # FIXME
-            oprime = self._make_zero_diff(o)
-        else:
+        if aprime.domains(): # TODO: Assuming this is equivalent to 'is_constant', which may not be the case...
             oprime = o.reconstruct(aprime)
+        else:
+            oprime = self._make_zero_diff(o)
         return (o, oprime)
 
 class UnimplementedADRules(object):
@@ -1050,6 +971,20 @@ class UnimplementedADRules(object):
         return (o, oprime)
 
     def determinant(self, o, a):
+        """FIXME: Some possible rules:
+
+        d detA / dv = detA * tr(inv(A) * dA/dv)
+
+        or
+
+        d detA / d row0 = cross(row1, row2)
+        d detA / d row1 = cross(row2, row0)
+        d detA / d row2 = cross(row0, row1)
+
+        i.e.
+
+        d detA / d A = [cross(row1, row2), cross(row2, row0), cross(row0, row1)] # or transposed or something
+        """
         error("Derivative of determinant not implemented, apply expand_compounds before AD.")
         A, Ap = a
         #oprime = ...
@@ -1067,6 +1002,13 @@ class UnimplementedADRules(object):
         0 = d/dx [Ainv*A] = Ainv' * A + Ainv * A'
         Ainv' * A = - Ainv * A'
         Ainv' = - Ainv * A' * Ainv
+
+        K J = I
+        d/dv[r] (K[i,j] J[j,k]) = d/dv[r] K[i,j] J[j,k] + K[i,j] d/dv[r] J[j,k] = 0
+        d/dv[r] K[i,j] J[j,k] = -K[i,j] d/dv[r] J[j,k]
+        d/dv[r] K[i,j] J[j,k] K[k,s] = -K[i,j] d/dv[r] J[j,k] K[k,s]
+        d/dv[r] K[i,j] I[j,s] = -K[i,j] (d/dv[r] J[j,k]) K[k,s]
+        d/dv[r] K[i,s] = -K[i,j] (d/dv[r] J[j,k]) K[k,s]
         """
         A, Ap = a
-        return (o, -o*Ap*o) # Any potential index problems here?
+        return (o, -o*Ap*o) # FIXME: Need reshaping move derivative axis to the end of this expression
