@@ -19,18 +19,17 @@ classes (functions), including TestFunction and TrialFunction."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2008-2009.
-#
-# First added:  2008-03-14
-# Last changed: 2011-10-20
 
 from ufl.log import deprecate
 from ufl.assertions import ufl_assert
-from ufl.terminal import FormArgument
+from ufl.core.terminal import Terminal, FormArgument
 from ufl.split_functions import split
 from ufl.finiteelement import FiniteElementBase
+from ufl.core.ufl_type import ufl_type
 
 # --- Class representing an argument (basis function) in a form ---
 
+@ufl_type()
 class Argument(FormArgument):
     """UFL value: Representation of an argument to a form."""
     __slots__ = ("_element", "_number", "_part", "_repr")
@@ -75,7 +74,8 @@ class Argument(FormArgument):
         ufl_assert(self.part() is None, "Deprecation transition for count() will not work with parts.")
         return self.number() # I think this will work ok in most cases during the deprecation transition
 
-    def shape(self):
+    @property
+    def ufl_shape(self):
         return self._element.value_shape()
 
     def is_cellwise_constant(self):
@@ -95,6 +95,13 @@ class Argument(FormArgument):
         s = self._element.signature_data(domain_numbering=domain_numbering)
         return ("Argument", self._number, self._part) + s
 
+    def signature_data(self, renumbering):
+        "Signature data for form arguments depend on the global numbering of the form arguments and domains."
+        edata = self.element().signature_data(renumbering)
+        d = self.domain()
+        ddata = None if d is None else d.signature_data(renumbering)
+        return ("Coefficient", self._number, self._part, edata, ddata)
+
     def __str__(self):
         number = str(self._number)
         if len(number) == 1:
@@ -129,7 +136,6 @@ class Argument(FormArgument):
                 self._part == other._part and
                 self._element == other._element)
 
-
 # --- Helper functions for pretty syntax ---
 
 def TestFunction(element, part=None):
