@@ -22,11 +22,13 @@
 # Modified by Marie E. Rognes 2012
 
 from collections import defaultdict
+from six import iteritems
 
+from ufl.corealg.traversal import traverse_unique_terminals
 from ufl.log import warning, error, deprecate
 from ufl.assertions import ufl_assert
 from ufl.common import istr, EmptyDict
-from ufl.terminal import Terminal
+from ufl.core.terminal import Terminal
 from ufl.protocols import id_or_none
 from ufl.cell import as_cell, affine_cells, Cell, ProductCell
 
@@ -94,10 +96,16 @@ class Domain(object):
             self._label = flat_domain.label()
             self._data = flat_domain.data()
 
-            # FIXME: Get geometric dimension from self._coordinates instead!
-            ufl_assert(self._coordinates.shape() == (self._cell.geometric_dimension(),),
-                       "Shape of coordinates %s does not match geometric dimension %d of cell." %\
-                (self._coordinates.shape(), self._cell.geometric_dimension()))
+            # Get geometric dimension from self._coordinates shape
+            gdim, = self._coordinates.ufl_shape
+            if gdim != self._cell.geometric_dimension():
+                warning("Using geometric dimension from coordinates!")
+                self._cell = Cell(self._cell.cellname(), gdim)
+            #ufl_assert(self._coordinates.ufl_shape == (self._cell.geometric_dimension(),),
+            #           "Shape of coordinates %s does not match geometric dimension %d of cell." %\
+            #    (self._coordinates.ufl_shape, self._cell.geometric_dimension()))
+        else:
+            ufl_error("Invalid first argument to Domain.")
 
         # Now we should have a Cell or something went wrong
         ufl_assert(isinstance(self._cell, Cell), "Failed to construct a Cell from input arguments.")
@@ -158,24 +166,33 @@ class Domain(object):
         "Return the coordinate vector field this domain is defined in terms of."
         return self._coordinates
 
+    def coordinate_element(self):
+        "Return the finite element of the coordinate vector field of this domain."
+        x = self.coordinates()
+        if x is None:
+            from ufl import VectorElement
+            return VectorElement("Lagrange", self, 1)
+        else:
+            return x.element()
+
     def label(self):
         "Return the label identifying this domain. None means no label has been set."
         return self._label
 
     def is_piecewise_linear_simplex_domain(self):
-        x = self.coordinates()
-        return (x is None or x.element().degree() == 1) and (self.cell().cellname() in affine_cells)
+        return (self.coordinate_element().degree() == 1) and (self.cell().cellname() in affine_cells)
 
     def data(self):
         "Return attached data object."
         return self._data
 
-    def signature_data(self, domain_numbering):
+    def signature_data(self, renumbering):
         "Signature data of domain depend on the global domain numbering."
-        c = self.cell()
-        key = (c, self.label())
-        data = (c, domain_numbering[key])
-        return data
+        count = renumbering[self]
+        cdata = self.cell()
+        x = self.coordinates()
+        xdata = (None if x is None else x.signature_data(renumbering))
+        return (count, cdata, xdata)
 
     def hash_data(self):
         # Including only id of data here.
@@ -202,7 +219,7 @@ class Domain(object):
         if self._coordinates is None:
             c = ""
         else:
-            c = "and coordinates %r" % self._coordinates
+            c = " and coordinates %r" % self._coordinates
         s = (self._cell, self._label, c)
         return "<Domain built from %s with label %s%s>" % s
 
@@ -311,7 +328,7 @@ def as_domain(domain):
 def join_subdomain_data(subdomain_datas): # FIXME: Remove? Think it's unused now.
     newdata = {}
     for data in subdomain_datas:
-        for k,v in data.iteritems():
+        for k, v in iteritems(data):
             nv = newdata.get(k)
             if nv is None:
                 # New item, just add it
@@ -328,13 +345,14 @@ def check_domain_compatibility(domains):
     labels = set(domain.label() for domain in domains)
     ufl_assert(len(labels) == 1 or (len(labels) == 2 and None in labels),
                "Got incompatible domain labels %s in check_domain_compatibility." % (labels,))
-    cell = domains[0].cell()
-    coordinates = domains[0].coordinates()
-    for dom in domains[1:]:
-        if dom.cell() != cell:
-            error("Cell mismatch between domains with same label.")
-        if dom.coordinates() != coordinates:
-            error("Coordinates mismatch between domains with same label.")
+
+    all_cellnames = [dom.cell().cellname() for dom in domains]
+    if len(set(all_cellnames)) != 1:
+        error("Cellname mismatch between domains with same label.")
+
+    all_coordinates = set(dom.coordinates() for dom in domains) - set((None,))
+    if len(all_coordinates) > 1:
+        error("Coordinates mismatch between domains with same label.")
 
 def join_domains(domains):
     """Take a list of Domains and return a list with only unique domain objects.
@@ -358,7 +376,7 @@ def join_domains(domains):
         elif len(label2domlist) == 2:
             none_domains = label2domlist[None]
             del label2domlist[None]
-            key, = label2domlist.keys()
+            key, = list(label2domlist.keys())
             label2domlist[key].extend(none_domains)
         else:
             error("Ambiguous mapping of domains with label None to multiple domains with different labels.")
@@ -415,7 +433,6 @@ def join_domains(domains):
     return tuple(newdomains)
 
 def extract_domains(expr):
-    from ufl.algorithms.traversal import traverse_unique_terminals
     domainlist = []
     for t in traverse_unique_terminals(expr):
         domainlist.extend(t.domains())
