@@ -21,9 +21,6 @@
 # Modified by Marie E. Rognes 2010, 2012
 # Modified by Andrew T. T. McRae 2014
 # Modified by Lawrence Mitchell 2014
-#
-# First added:  2008-03-03
-# Last changed: 2014-03-18
 
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
 from ufl.finiteelement.finiteelement import FiniteElement
@@ -36,3 +33,7 @@ from ufl.finiteelement.tensorproductelement import TensorProductElement
 from ufl.finiteelement.outerproductelement import OuterProductElement
 from ufl.finiteelement.outerproductelement import OuterProductVectorElement
 from ufl.finiteelement.hdivcurl import HDiv, HCurl
+from ufl.finiteelement.brokenelement import BrokenElement
+from ufl.finiteelement.traceelement import TraceElement
+from ufl.finiteelement.facetelement import FacetElement
+from ufl.finiteelement.interiorelement import InteriorElement
