@@ -20,9 +20,12 @@ elements by calling the function register_element."""
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Marie E. Rognes <meg@simula.no>, 2010
+# Modified by Lizao Li <lzlarryli@gmail.com>, 2015
+
+from __future__ import print_function
 
 from ufl.assertions import ufl_assert
-from ufl.sobolevspace import L2, H1, H2, HDiv, HCurl
+from ufl.sobolevspace import L2, H1, H2, HDiv, HCurl, HEin
 from ufl.common import istr
 
 # List of valid elements
@@ -47,15 +50,22 @@ def register_alias(alias, to):
     aliases[alias] = to
 
 def show_elements():
-    print "Showing all registered elements:"
+    print("Showing all registered elements:")
+    print("================================")
+    shown = set()
     for k in sorted(ufl_elements.keys()):
-        (family, short_name, sobolev_space, mapping, degree_range, cellnames) = ufl_elements[k]
-        print
-        print "Finite Element Family: %s, %s" % (repr(family), repr(short_name))
-        print "Sobolev space: %s" % (sobolev_space,)
-        print "Mapping: %s" % (mapping,)
-        print "Degree range: ", degree_range
-        print "Defined on cellnames:" , cellnames
+        data = ufl_elements[k]
+        if data in shown:
+            continue
+        shown.add(data)
+        (family, short_name, value_rank, sobolev_space, mapping, degree_range, cellnames) = data
+        print("Finite element family: %s, %s" % (repr(family), repr(short_name)))
+        print("Sobolev space: %s" % (sobolev_space,))
+        print("Mapping: %s" % (mapping,))
+        print("Degree range: %s" % (degree_range,))
+        print("Value rank: %s" % (value_rank,))
+        print("Defined on cellnames: %s" % (cellnames,))
+        print()
 
 # FIXME: Consider cleanup of element names. Use notation from periodic table as the main, keep old names as compatibility aliases.
 
@@ -75,18 +85,17 @@ def show_elements():
 simplices = ("interval", "triangle", "tetrahedron")
 cubes     = ("interval", "quadrilateral", "hexahedron")
 any_cell  = (None,
-             "cell0D", "cell1D", "cell2D", "cell3D",
              "vertex", "interval",
              "triangle", "tetrahedron",
              "quadrilateral", "hexahedron")
 
 # Elements in the periodic table # TODO: Register these as aliases of periodic table element description instead of the other way around
-register_element("Lagrange", "CG",                       0, H1,    "identity", (1, None), any_cell)                   # "P"
-register_element("Brezzi-Douglas-Marini", "BDM",         1, HDiv,  "contravariant Piola", (1, None), simplices[1:]) # "BDMf" (2d), "N2f" (3d)
-register_element("Discontinuous Lagrange", "DG",         0, L2,    "identity", (0, None), any_cell)                   # "DG"
-register_element("Nedelec 1st kind H(curl)", "N1curl",   1, HCurl, "covariant Piola", (1, None), simplices[1:])     # "RTe" (2d), "N1e" (3d)
-register_element("Nedelec 2nd kind H(curl)", "N2curl",   1, HCurl, "covariant Piola", (1, None), simplices[1:])     # "BDMe" (2d), "N2e" (3d)
-register_element("Raviart-Thomas", "RT",                 1, HDiv,  "contravariant Piola", (1, None), simplices[1:]) # "RTf" , "N1f" (3d)
+register_element("Lagrange", "CG",                       0, H1,    "identity", (1, None), any_cell)                  # "P"
+register_element("Brezzi-Douglas-Marini", "BDM",         1, HDiv,  "contravariant Piola", (1, None), simplices[1:])  # "BDMF" (2d), "N2F" (3d)
+register_element("Discontinuous Lagrange", "DG",         0, L2,    "identity", (0, None), any_cell)                  # "DP"
+register_element("Nedelec 1st kind H(curl)", "N1curl",   1, HCurl, "covariant Piola", (1, None), simplices[1:])      # "RTE"  (2d), "N1E" (3d)
+register_element("Nedelec 2nd kind H(curl)", "N2curl",   1, HCurl, "covariant Piola", (1, None), simplices[1:])      # "BDME" (2d), "N2E" (3d)
+register_element("Raviart-Thomas", "RT",                 1, HDiv,  "contravariant Piola", (1, None), simplices[1:])  # "RTF"  (2d), "N1F" (3d)
 
 # Elements not in the periodic table
 register_element("Argyris", "ARG",                       0, H2,   "identity", (1, None), simplices[1:])
@@ -103,10 +112,12 @@ register_element("Morley", "MOR",                        0, H2,   "identity", No
 register_element("Boundary Quadrature", "BQ", 0, L2, "identity", (0, None), any_cell)
 register_element("Bubble", "B",               0, H1, "identity", (2, None), simplices)
 register_element("Quadrature", "Quadrature",  0, L2, "identity", (0, None), any_cell)
-register_element("Real", "R",                 0, L2, "identity", (0, 0),    any_cell)
+register_element("Real", "R",                 0, L2, "identity", (0, 0),    any_cell + ("OuterProductCell",))
 register_element("Undefined", "U",            0, L2, "identity", (0, None), any_cell)
 register_element("Lobatto", "Lob",            0, L2, "identity", (1, None), ("interval",))
 register_element("Radau",   "Rad",            0, L2, "identity", (0, None), ("interval",))
+register_element("Discontinuous Lagrange Trace", "DGT", 0, L2, "identity", (0, None), any_cell)
+register_element("Regge", "Regge",            2, HEin, "pullback as metric", (0, None), simplices[1:])
 
 # Let Nedelec H(div) elements be aliases to BDMs/RTs
 register_alias("Nedelec 1st kind H(div)",
@@ -121,31 +132,31 @@ register_alias("N2div",
 
 # New elements introduced for the periodic table 2014
 register_element2("Q",     0, H1,    "identity",            (1, None), cubes)
-register_element2("DGQ",   0, L2,    "identity",            (0, None), cubes)
-register_element2("RTQe",  1, HCurl, "covariant Piola",     (1, None), ("quadrilateral",))
-register_element2("RTQf",  1, HDiv,  "contravariant Piola", (1, None), ("quadrilateral",))
-register_element2("NQe",   1, HCurl, "covariant Piola",     (1, None), ("hexahedron",))
-register_element2("NQf",   1, HDiv,  "contravariant Piola", (1, None), ("hexahedron",))
+register_element2("DQ",    0, L2,    "identity",            (0, None), cubes)
+register_element2("RTCE",  1, HCurl, "covariant Piola",     (1, None), ("quadrilateral",))
+register_element2("RTCF",  1, HDiv,  "contravariant Piola", (1, None), ("quadrilateral",))
+register_element2("NCE",   1, HCurl, "covariant Piola",     (1, None), ("hexahedron",))
+register_element2("NCF",   1, HDiv,  "contravariant Piola", (1, None), ("hexahedron",))
 
 register_element2("S",     0, H1,    "identity",            (1, None), cubes)
-register_element2("DGS",   0, L2,    "identity",            (1, None), cubes)
-register_element2("BDMSe", 1, HCurl, "covariant Piola",     (1, None), ("quadrilateral",))
-register_element2("BDMSf", 1, HDiv,  "contravariant Piola", (1, None), ("quadrilateral",))
-register_element2("AAe",   1, HCurl, "covariant Piola",     (1, None), ("hexahedron",))
-register_element2("AAf",   1, HDiv,  "contravariant Piola", (1, None), ("hexahedron",))
+register_element2("DPC",   0, L2,    "identity",            (1, None), cubes)
+register_element2("BDMCE", 1, HCurl, "covariant Piola",     (1, None), ("quadrilateral",))
+register_element2("BDMCF", 1, HDiv,  "contravariant Piola", (1, None), ("quadrilateral",))
+register_element2("AAE",   1, HCurl, "covariant Piola",     (1, None), ("hexahedron",))
+register_element2("AAF",   1, HDiv,  "contravariant Piola", (1, None), ("hexahedron",))
 
 # New aliases introduced for the periodic table 2014
 register_alias("P",    lambda family, dim, order, degree: ("Lagrange",                 order))
-register_alias("RTe",  lambda family, dim, order, degree: ("Nedelec 1st kind H(curl)", order))
-register_alias("RTf",  lambda family, dim, order, degree: ("Raviart-Thomas",           order))
-register_alias("N1e",  lambda family, dim, order, degree: ("Nedelec 1st kind H(curl)", order))
-register_alias("N1f",  lambda family, dim, order, degree: ("Raviart-Thomas",           order))
-
-register_alias("BDMe", lambda family, dim, order, degree: ("Nedelec 2nd kind H(curl)", order))
-register_alias("BDMf", lambda family, dim, order, degree: ("Brezzi-Douglas-Marini",    order))
-register_alias("N2e",  lambda family, dim, order, degree: ("Nedelec 2nd kind H(curl)", order))
-register_alias("N2f",  lambda family, dim, order, degree: ("Brezzi-Douglas-Marini",    order))
+register_alias("DP",   lambda family, dim, order, degree: ("Discontinuous Lagrange",   order))
+register_alias("RTE",  lambda family, dim, order, degree: ("Nedelec 1st kind H(curl)", order))
+register_alias("RTF",  lambda family, dim, order, degree: ("Raviart-Thomas",           order))
+register_alias("N1E",  lambda family, dim, order, degree: ("Nedelec 1st kind H(curl)", order))
+register_alias("N1F",  lambda family, dim, order, degree: ("Raviart-Thomas",           order))
 
+register_alias("BDME", lambda family, dim, order, degree: ("Nedelec 2nd kind H(curl)", order))
+register_alias("BDMF", lambda family, dim, order, degree: ("Brezzi-Douglas-Marini",    order))
+register_alias("N2E",  lambda family, dim, order, degree: ("Nedelec 2nd kind H(curl)", order))
+register_alias("N2F",  lambda family, dim, order, degree: ("Brezzi-Douglas-Marini",    order))
 
 # Finite element exterior calculus notation
 def feec_element(family, n, r, k):
@@ -153,38 +164,53 @@ def feec_element(family, n, r, k):
     # r = polynomial order
     # k = form_degree
 
-    # mapping from (feec name, domain dimension, form degree) to (family name, polynomial order)
+    # Note: We always map to edge elements in 2D, don't know how to differentiate otherwise?
+
+    # Mapping from (feec name, domain dimension, form degree) to (family name, polynomial order)
     _feec_elements = {
         "P- Lambda": (
-            (("P", r), ("DG", r - 1)),
-            (("P", r), ("RTe", r),      ("DG", r - 1)),
-            (("P", r), ("N1e", r),      ("N1f", r),   ("DG", r - 1)),
+            (("P", r), ("DP", r - 1)),
+            (("P", r), ("RTE", r),      ("DP", r - 1)),
+            (("P", r), ("N1E", r),      ("N1F", r),   ("DP", r - 1)),
             ),
         "P Lambda": (
-            (("P", r), ("DG", r)),
-            (("P", r), ("BDMe", r),     ("DG", r)),
-            (("P", r), ("N2e", r),      ("N2f", r),   ("DG", r)),
+            (("P", r), ("DP", r)),
+            (("P", r), ("BDME", r),     ("DP", r)),
+            (("P", r), ("N2E", r),      ("N2F", r),   ("DP", r)),
             ),
         "Q- Lambda": (
-            (("Q", r), ("DGQ", r - 1)),
-            (("Q", r), ("RTQe", r),     ("DGQ", r - 1)),
-            (("Q", r), ("NQe", r),      ("NQf", r),   ("DGQ", r - 1)),
+            (("Q", r), ("DQ", r - 1)),
+            (("Q", r), ("RTCE", r),     ("DQ", r - 1)),
+            (("Q", r), ("NCE", r),      ("NCF", r),   ("DQ", r - 1)),
             ),
         "S Lambda": (
-            (("S", r), ("DGS", r)),
-            (("S", r), ("BDMSe", r),    ("DGS", r)),
-            (("S", r), ("AAe", r),      ("AAf", r),   ("DGS", r)),
+            (("S", r), ("DPC", r)),
+            (("S", r), ("BDMCE", r),    ("DPC", r)),
+            (("S", r), ("AAE", r),      ("AAF", r),   ("DPC", r)),
             ),
         }
 
+    # New notation, old verbose notation (including "Lambda") might be removed
+    _feec_elements["P-"] = _feec_elements["P- Lambda"]
+    _feec_elements["P"]  = _feec_elements["P Lambda"]
+    _feec_elements["Q-"] = _feec_elements["Q- Lambda"]
+    _feec_elements["S"]  = _feec_elements["S Lambda"]
+
     family, r = _feec_elements[family][n - 1][k]
+
     return family, r
 
+# General FEEC notation, old verbose (can be removed)
 register_alias("P- Lambda", lambda family, dim, order, degree: feec_element(family, dim, order, degree))
 register_alias("P Lambda",  lambda family, dim, order, degree: feec_element(family, dim, order, degree))
 register_alias("Q- Lambda", lambda family, dim, order, degree: feec_element(family, dim, order, degree))
 register_alias("S Lambda",  lambda family, dim, order, degree: feec_element(family, dim, order, degree))
 
+# General FEEC notation, new compact notation
+register_alias("P-", lambda family, dim, order, degree: feec_element(family, dim, order, degree))
+#register_alias("P Lambda",  lambda family, dim, order, degree: feec_element(family, dim, order, degree))
+register_alias("Q-", lambda family, dim, order, degree: feec_element(family, dim, order, degree))
+#register_alias("S Lambda",  lambda family, dim, order, degree: feec_element(family, dim, order, degree))
 
 def canonical_element_description(family, cell, order, form_degree):
     """Given basic element information, return corresponding element information on canonical form.
@@ -206,6 +232,10 @@ def canonical_element_description(family, cell, order, form_degree):
         gdim = None
         cellname = None
 
+    # Catch general FEEC notation "P" and "S"
+    if form_degree is not None and family in ("P", "S"):
+        family, order = feec_element(family, tdim, order, form_degree)
+
     # Check whether this family is an alias for something else
     while family in aliases:
         ufl_assert(tdim is not None, "Need dimension to handle element aliases.")
@@ -241,8 +271,13 @@ def canonical_element_description(family, cell, order, form_degree):
     # Override sobolev_space for piecewise constants (TODO: necessary?)
     if order == 0:
         sobolev_space = L2
-
-    if value_rank == 1:
+    if value_rank == 2:
+        # Tensor valued fundamental elements in HEin have this shape
+        ufl_assert(gdim is not None and tdim is not None,
+               "Cannot infer shape of element without topological and geometric dimensions.")
+        reference_value_shape = (tdim, tdim)
+        value_shape = (gdim, gdim)
+    elif value_rank == 1:
         # Vector valued fundamental elements in HDiv and HCurl have a shape
         ufl_assert(gdim != None and tdim != None,
                "Cannot infer shape of element without topological and geometric dimensions.")
