@@ -19,14 +19,11 @@
 #
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
-#
-# First added:  2008-03-03
-# Last changed: 2012-08-16
 
-from itertools import izip
+from six.moves import zip
 from ufl.assertions import ufl_assert
 from ufl.permutation import compute_indices
-from ufl.common import product, index_to_component, component_to_index, istr, EmptyDict
+from ufl.common import product, istr, EmptyDict
 from ufl.log import info_blue, warning, warning_blue, error
 
 from ufl.finiteelement.finiteelementbase import FiniteElementBase
@@ -44,7 +41,8 @@ class EnrichedElement(FiniteElementBase):
                    "Domain mismatch for sub elements of enriched element.")
 
         if isinstance(elements[0].degree(), int):
-            degree = max(e.degree() for e in elements)
+            degrees = { e.degree() for e in elements } - { None }
+            degree = max(degrees) if degrees else None
         else:
             degree = tuple(map(max, zip(*[e.degree() for e in elements])))
 
@@ -59,12 +57,20 @@ class EnrichedElement(FiniteElementBase):
         ufl_assert(all(e.value_shape() == value_shape for e in elements),
                    "Element value shape mismatch.")
 
+        reference_value_shape = elements[0].reference_value_shape()
+        ufl_assert(all(e.reference_value_shape() == reference_value_shape for e in elements),
+                   "Element reference value shape mismatch.")
+
+        #mapping = elements[0].mapping() # FIXME: This fails for a mixed subelement here.
+        #ufl_assert(all(e.mapping() == mapping for e in elements),
+        #           "Element mapping mismatch.")
+
         # Initialize element data
-        super(EnrichedElement, self).__init__("EnrichedElement", domain, degree,
-                                              quad_scheme, value_shape)
+        FiniteElementBase.__init__(self, "EnrichedElement", domain, degree,
+                                   quad_scheme, value_shape, reference_value_shape)
 
         # Cache repr string
-        self._repr =  "EnrichedElement(*%r)" % ([repr(e) for e in self._elements],)
+        self._repr = "EnrichedElement(*%r)" % ([repr(e) for e in self._elements],)
 
     def reconstruction_signature(self):
         """Format as string for evaluation as Python object.
@@ -81,7 +87,7 @@ class EnrichedElement(FiniteElementBase):
         """Construct a new EnrichedElement object with some properties
         replaced with new values."""
         elements = [e.reconstruct(**kwargs) for e in self._elements]
-        if all(a == b for (a,b) in izip(elements, self._elements)):
+        if all(a == b for (a, b) in zip(elements, self._elements)):
             return self
         return EnrichedElement(*elements)
 
@@ -90,6 +96,9 @@ class EnrichedElement(FiniteElementBase):
         element is spatially constant over each cell."""
         return all(e.is_cellwise_constant() for e in self._elements)
 
+    def mapping(self):
+        return self._elements[0].mapping()
+
     def __str__(self):
         "Format as string for pretty printing."
         return "<%s>" % " + ".join(str(e) for e in self._elements)
@@ -98,7 +107,7 @@ class EnrichedElement(FiniteElementBase):
         "Format as string for pretty printing."
         return "<%s>" % " + ".join(e.shortstr() for e in self._elements)
 
-    def signature_data(self, domain_numbering):
-        data = ("EnrichedElement", 
-                tuple(e.signature_data(domain_numbering=domain_numbering) for e in self._elements))
+    def signature_data(self, renumbering):
+        data = ("EnrichedElement",
+                tuple(e.signature_data(renumbering) for e in self._elements))
         return data
