@@ -20,14 +20,17 @@
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
 # Modified by Anders Logg 2014
-#
-# First added:  2008-03-03
-# Last changed: 2014-02-24
 
-from itertools import izip, chain
+from itertools import chain
+
+from six import iteritems
+from six.moves import zip
+from six.moves import xrange as range
+
 from ufl.assertions import ufl_assert
 from ufl.permutation import compute_indices
-from ufl.common import product, index_to_component, component_to_index, istr, EmptyDict
+from ufl.common import product, istr, EmptyDict
+from ufl.utils.indexflattening import flatten_multiindex, unflatten_index, shape_to_strides
 from ufl.geometry import as_domain
 from ufl.log import info_blue, warning, warning_blue, error
 
@@ -46,7 +49,7 @@ class MixedElement(FiniteElementBase):
         if len(elements) == 1 and isinstance(elements[0], (tuple, list)):
             elements = elements[0]
         # Interpret nested tuples as sub-mixedelements recursively
-        elements = [MixedElement(e) if isinstance(e, (tuple,list)) else e
+        elements = [MixedElement(e) if isinstance(e, (tuple, list)) else e
                     for e in elements]
         self._sub_elements = elements
 
@@ -88,10 +91,14 @@ class MixedElement(FiniteElementBase):
                 "Provided value_shape doesn't match the total "\
                 "value size of all subelements.")
 
+        # Always use a flat reference value shape
+        reference_value_shape = (sum(product(s.reference_value_shape()) for s in self._sub_elements),)
+
         # Initialize element data
-        degree = max(e.degree() for e in self._sub_elements)
-        super(MixedElement, self).__init__("Mixed", domain, degree,
-                                           quad_scheme, value_shape)
+        degrees = { e.degree() for e in self._sub_elements } - { None }
+        degree = max(degrees) if degrees else None
+        FiniteElementBase.__init__(self, "Mixed", domain, degree, quad_scheme,
+                                   value_shape, reference_value_shape)
 
         # Cache repr string
         self._repr = "MixedElement(*%r, **{'value_shape': %r })" %\
@@ -124,10 +131,10 @@ class MixedElement(FiniteElementBase):
 
     def reconstruct_from_elements(self, *elements):
         "Reconstruct a mixed element from new subelements."
-        if all(a == b for (a,b) in izip(elements, self._sub_elements)):
+        if all(a == b for (a, b) in zip(elements, self._sub_elements)):
             return self
         ufl_assert(all(a.value_shape() == b.value_shape()
-                       for (a,b) in izip(elements, self._sub_elements)),
+                       for (a, b) in zip(elements, self._sub_elements)),
             "Expecting new elements to have same value shape as old ones.")
         return MixedElement(*elements, value_shape=self.value_shape())
 
@@ -141,10 +148,11 @@ class MixedElement(FiniteElementBase):
         j = 0
         for e in self._sub_elements:
             sh = e.value_shape()
+            st = shape_to_strides(sh)
             # Map symmetries of subelement into index space of this element
-            for c0, c1 in e.symmetry().iteritems():
-                j0 = component_to_index(c0, sh) + j
-                j1 = component_to_index(c1, sh) + j
+            for c0, c1 in iteritems(e.symmetry()):
+                j0 = flatten_multiindex(c0, st) + j
+                j1 = flatten_multiindex(c1, st) + j
                 sm[(j0,)] = (j1,)
             # Update base index for next element
             j += product(sh)
@@ -152,6 +160,9 @@ class MixedElement(FiniteElementBase):
                    "Size mismatch in symmetry algorithm.")
         return sm or EmptyDict
 
+    def mapping(self):
+        error("The mapping of a mixed element is not defined. Inspect subelements instead.")
+
     def num_sub_elements(self):
         "Return number of sub elements."
         return len(self._sub_elements)
@@ -160,18 +171,6 @@ class MixedElement(FiniteElementBase):
         "Return list of sub elements."
         return self._sub_elements
 
-    def num_mixed_sub_elements(self):
-        "Return number of mixed sub elements."
-        # FIXME: Use this where intended, for disambiguation
-        #        w.r.t. different sub_elements meanings.
-        return len(self._sub_elements)
-
-    def mixed_sub_elements(self):
-        "Return list of mixed sub elements."
-        # FIXME: Use this where intended, for disambiguation
-        #        w.r.t. different sub_elements meanings.
-        return self._sub_elements
-
     def extract_subelement_component(self, i):
         """Extract direct subelement index and subelement relative
         component index for a given component index"""
@@ -185,7 +184,7 @@ class MixedElement(FiniteElementBase):
             j, = i
 
             # Find subelement for this index
-            for k, e in enumerate(self._sub_elements):
+            for sub_element_index, e in enumerate(self._sub_elements):
                 sh = e.value_shape()
                 si = product(sh)
                 if j < si:
@@ -194,21 +193,54 @@ class MixedElement(FiniteElementBase):
             ufl_assert(j >= 0, "Moved past last value component!")
 
             # Convert index into a shape tuple
-            j = index_to_component(j, sh)
+            st = shape_to_strides(sh)
+            component = unflatten_index(j, st)
         else:
             # Indexing into a multidimensional tensor
             # where subelement index is first axis
-            k = i[0]
-            ufl_assert(k < len(self._sub_elements),
-                       "Illegal component index (dimension %d)." % k)
-            j = i[1:]
-        return (k, j)
+            sub_element_index = i[0]
+            ufl_assert(sub_element_index < len(self._sub_elements),
+                       "Illegal component index (dimension %d)." % sub_element_index)
+            component = i[1:]
+        return (sub_element_index, component)
 
     def extract_component(self, i):
         """Recursively extract component index relative to a (simple) element
         and that element for given value component index"""
-        k, j = self.extract_subelement_component(i)
-        return self._sub_elements[k].extract_component(j)
+        sub_element_index, component = self.extract_subelement_component(i)
+        return self._sub_elements[sub_element_index].extract_component(component)
+
+    def extract_subelement_reference_component(self, i):
+        """Extract direct subelement index and subelement relative
+        reference_component index for a given reference_component index"""
+        if isinstance(i, int):
+            i = (i,)
+        self._check_reference_component(i)
+
+        # Select between indexing modes
+        assert len(self.reference_value_shape()) == 1
+        # Indexing into a long vector of flattened subelement shapes
+        j, = i
+
+        # Find subelement for this index
+        for sub_element_index, e in enumerate(self._sub_elements):
+            sh = e.reference_value_shape()
+            si = product(sh)
+            if j < si:
+                break
+            j -= si
+        ufl_assert(j >= 0, "Moved past last value reference_component!")
+
+        # Convert index into a shape tuple
+        st = shape_to_strides(sh)
+        reference_component = unflatten_index(j, st)
+        return (sub_element_index, reference_component)
+
+    def extract_reference_component(self, i):
+        """Recursively extract reference_component index relative to a (simple) element
+        and that element for given value reference_component index"""
+        sub_element_index, reference_component = self.extract_subelement_reference_component(i)
+        return self._sub_elements[sub_element_index].extract_reference_component(reference_component)
 
     def is_cellwise_constant(self, component=None):
         """Return whether the basis functions of this
@@ -237,8 +269,9 @@ class MixedElement(FiniteElementBase):
             i, e = self.extract_component(component)
             return e.degree()
 
-    def signature_data(self, domain_numbering):
-        data = ("MixedElement", self._value_shape, tuple(e.signature_data(domain_numbering=domain_numbering) for e in self._sub_elements))
+    def signature_data(self, renumbering):
+        data = ("MixedElement", self._value_shape,
+                tuple(e.signature_data(renumbering) for e in self._sub_elements))
         return data
 
     def __str__(self):
@@ -298,7 +331,7 @@ class VectorElement(MixedElement):
         value_shape = shape + sub_element.value_shape()
 
         # Initialize element data
-        super(VectorElement, self).__init__(sub_elements, value_shape=value_shape)
+        MixedElement.__init__(self, sub_elements, value_shape=value_shape)
         self._family = family
         self._degree = degree
         self._sub_element = sub_element
@@ -309,9 +342,12 @@ class VectorElement(MixedElement):
             (self._family, self.domain(), self._degree,
              len(self._sub_elements), self._quad_scheme)
 
-    def signature_data(self, domain_numbering):
+    def mapping(self):
+        return self._sub_element.mapping()
+
+    def signature_data(self, renumbering):
         data = ("VectorElement", self._family, self._degree, len(self._sub_elements), self._quad_scheme, self._form_degree,
-                ("no domain" if self._domain is None else self._domain.signature_data(domain_numbering=domain_numbering)))
+                ("no domain" if self._domain is None else self._domain.signature_data(renumbering)))
         return data
 
     def reconstruction_signature(self):
@@ -367,12 +403,12 @@ class TensorElement(MixedElement):
         if symmetry == True:
             ufl_assert(len(shape) == 2 and shape[0] == shape[1],
                        "Cannot set automatic symmetry for non-square tensor.")
-            symmetry = dict( ((i,j), (j,i)) for i in range(shape[0])
+            symmetry = dict( ((i, j), (j, i)) for i in range(shape[0])
                              for j in range(shape[1]) if i > j )
 
         # Validate indices in symmetry dict
         if isinstance(symmetry, dict):
-            for i,j in symmetry.iteritems():
+            for i, j in iteritems(symmetry):
                 ufl_assert(len(i) == len(j),
                            "Non-matching length of symmetry index tuples.")
                 for k in range(len(i)):
@@ -380,7 +416,8 @@ class TensorElement(MixedElement):
                                i[k] < shape[k] and j[k] < shape[k],
                                "Symmetry dimensions out of bounds.")
         else:
-            ufl_assert(symmetry is None, "Expecting symmetry to be None, True, or dict.")
+            ufl_assert(symmetry is None, "Expecting symmetry to be None (unset), True, or dict.")
+            symmetry = EmptyDict
 
         # Compute all index combinations for given shape
         indices = compute_indices(shape)
@@ -391,14 +428,14 @@ class TensorElement(MixedElement):
         sub_elements = []
         sub_element_mapping = {}
         for index in indices:
-            if symmetry and index in symmetry:
+            if index in symmetry:
                 continue
             sub_element_mapping[index] = len(sub_elements)
             sub_elements += [sub_element]
 
         # Update mapping for symmetry
         for index in indices:
-            if symmetry and index in symmetry:
+            if index in symmetry:
                 sub_element_mapping[index] = sub_element_mapping[symmetry[index]]
 
         # Get common family name (checked in FiniteElement.__init__)
@@ -408,7 +445,7 @@ class TensorElement(MixedElement):
         value_shape = shape + sub_element.value_shape()
 
         # Initialize element data
-        super(TensorElement, self).__init__(sub_elements, value_shape=value_shape)
+        MixedElement.__init__(self, sub_elements, value_shape=value_shape)
         self._family = family
         self._degree = degree
         self._sub_element = sub_element
@@ -421,9 +458,12 @@ class TensorElement(MixedElement):
             (self._family, self.domain(), self._degree, self._shape,
              self._symmetry, self._quad_scheme)
 
-    def signature_data(self, domain_numbering):
+    def mapping(self):
+        return self._sub_element.mapping()
+
+    def signature_data(self, renumbering):
         data = ("TensorElement", self._family, self._degree, self._shape, repr(self._symmetry), self._quad_scheme,
-                ("no domain" if self._domain is None else self._domain.signature_data(domain_numbering=domain_numbering)))
+                ("no domain" if self._domain is None else self._domain.signature_data(renumbering)))
         return data
 
     def reconstruction_signature(self):
@@ -473,26 +513,24 @@ class TensorElement(MixedElement):
     def symmetry(self):
         """Return the symmetry dict, which is a mapping c0 -> c1
         meaning that component c0 is represented by component c1."""
-        return self._symmetry or EmptyDict
+        return self._symmetry
 
     def __str__(self):
         "Format as string for pretty printing."
-        sym = ""
-        if isinstance(self._symmetry, dict):
-            tmp = ", ".join("%s -> %s" % (a,b) for (a,b) in self._symmetry.iteritems())
+        if self._symmetry:
+            tmp = ", ".join("%s -> %s" % (a, b) for (a, b) in iteritems(self._symmetry))
             sym = " with symmetries (%s)" % tmp
-        elif self._symmetry:
-            sym = " with symmetry"
+        else:
+            sym = ""
         return "<%s tensor element of degree %s and shape %s on a %s%s>" % \
             (self.family(), istr(self.degree()), self.value_shape(), self.domain(), sym)
 
     def shortstr(self):
         "Format as string for pretty printing."
-        sym = ""
-        if isinstance(self._symmetry, dict):
-            tmp = ", ".join("%s -> %s" % (a,b) for (a,b) in self._symmetry.iteritems())
+        if self._symmetry:
+            tmp = ", ".join("%s -> %s" % (a, b) for (a, b) in iteritems(self._symmetry))
             sym = " with symmetries (%s)" % tmp
-        elif self._symmetry:
-            sym = " with symmetry"
+        else:
+            sym = ""
         return "Tensor<%s x %s%s>" % (self.value_shape(),
                                       self._sub_element.shortstr(), sym)
