@@ -19,14 +19,10 @@
 #
 # Modified by Kristian B. Oelgaard
 # Modified by Marie E. Rognes 2010, 2012
-#
-# First added:  2008-03-03
-# Last changed: 2012-08-16
 
-from itertools import izip
 from ufl.assertions import ufl_assert
 from ufl.permutation import compute_indices
-from ufl.common import product, index_to_component, component_to_index, istr, EmptyDict
+from ufl.common import product, istr, EmptyDict
 from ufl.geometry import as_domain, as_cell, ProductCell, ProductDomain
 from ufl.log import info_blue, warning, warning_blue, error
 
@@ -57,7 +53,8 @@ class TensorProductElement(FiniteElementBase):
         domain = ProductDomain([e.domain() for e in self._sub_elements])
 
         # Define polynomial degree as the maximal of each subelement
-        degree = max(e.degree() for e in self._sub_elements)
+        degrees = { e.degree() for e in self._sub_elements } - { None }
+        degree = max(degrees) if degrees else None
 
         # No quadrature scheme defined
         quad_scheme = None
@@ -66,12 +63,13 @@ class TensorProductElement(FiniteElementBase):
         # shape, and use this.
         # TODO: Not sure if this makes sense, what kind of product is used to build the basis?
         value_shape = self._sub_elements[0].value_shape()
+        reference_value_shape = self._sub_elements[0].reference_value_shape()
         ufl_assert(all(e.value_shape() == value_shape
                        for e in self._sub_elements),
                    "All subelements in must have same value shape")
 
-        super(TensorProductElement, self).__init__(family, domain, degree,
-                                                   quad_scheme, value_shape)
+        FiniteElementBase.__init__(self, family, domain, degree,
+                                   quad_scheme, value_shape, reference_value_shape)
 
     def reconstruction_signature(self):
         """Format as string for evaluation as Python object.
@@ -83,29 +81,16 @@ class TensorProductElement(FiniteElementBase):
         label and data, which must be reconstructed or supplied by other means.
         """
         return "TensorProductElement(%s)" % (', '.join(e.reconstruction_signature() for e in self._sub_elements),)
-        
+
+    def mapping(self):
+        error("The mapping of a mixed element is not defined. Inspect subelements instead.")
+
     def num_sub_elements(self):
         "Return number of subelements."
         return len(self._sub_elements)
 
     def sub_elements(self):
         "Return subelements (factors)."
-        # TODO: I don't think the concept of sub elements is quite well defined across
-        # all the current element types. Need to investigate how sub_elements is used in
-        # existing code, and eventually redesign here, hopefully just adding another function.
-        # Summary of different behaviours:
-        # - In MixedElement and subclasses each subelement corresponds to different value components.
-        # - In EnrichedElement sub_elements returns [] even though it contains multiple "children".
-        # - In RestrictedElement it returns the sub elements of its single children.
-        # - Here in TensorProductElement it returns sub elements that correspond to factors, not components.
-        return self._sub_elements
-
-    def num_tensorproduct_sub_elements(self): # FIXME: Use this where intended, for disambiguation w.r.t. different sub_elements meanings.
-        "Return number of tensorproduct sub elements."
-        return len(self._sub_elements)
-
-    def tensorproduct_sub_elements(self): # FIXME: Use this where intended, for disambiguation w.r.t. different sub_elements meanings.
-        "Return list of tensorproduct sub elements."
         return self._sub_elements
 
     def __str__(self):
@@ -118,7 +103,7 @@ class TensorProductElement(FiniteElementBase):
         return "TensorProductElement(%s)" \
             % str([e.shortstr() for e in self.sub_elements()])
 
-    def signature_data(self, domain_numbering):
-        data = ("TensorProductElement", 
-                tuple(e.signature_data(domain_numbering=domain_numbering) for e in self._sub_elements))
+    def signature_data(self, renumbering):
+        data = ("TensorProductElement",
+                tuple(e.signature_data(renumbering) for e in self._sub_elements))
         return data
