@@ -77,6 +77,10 @@ class Form(object):
         "_coefficient_numbering",
         "_hash",
         "_signature",
+        # --- Dict that external frameworks can place framework-specific
+        #     data in to be carried with the form
+        #     Never use this internally in ufl!
+        "_cache",
         )
 
     def __init__(self, integrals):
@@ -103,6 +107,9 @@ class Form(object):
         self._hash = None
         self._signature = None
 
+        # Never use this internally in ufl!
+        self._cache = {}
+
     # --- Accessor interface ---
 
     def integrals(self):
