@@ -23,9 +23,10 @@
 import math
 from ufl.log import warning, error
 from ufl.assertions import ufl_assert
-from ufl.operatorbase import Operator
+from ufl.core.operator import Operator
 from ufl.constantvalue import is_true_ufl_scalar, ScalarValue, Zero, FloatValue, IntValue, as_ufl
 from ufl.common import EmptyDict
+from ufl.core.ufl_type import ufl_type
 
 """
 TODO: Include additional functions available in <cmath> (need derivatives as well):
@@ -50,30 +51,18 @@ Implementation in C++ std::tr1:: or boost::math::tr1::
 
 #--- Function representations ---
 
+@ufl_type(is_abstract=True, is_scalar=True, num_ops=1)
 class MathFunction(Operator):
-    "Base class for all math functions"
+    "Base class for all unary scalar math functions."
     # Freeze member variables for objects in this class
-    __slots__ = ("_name", "_argument",)
+    __slots__ = ("_name",)
     def __init__(self, name, argument):
-        Operator.__init__(self)
+        Operator.__init__(self, (argument,))
         ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
         self._name     = name
-        self._argument = argument
-
-    def operands(self):
-        return (self._argument,)
-
-    def free_indices(self):
-        return ()
-
-    def index_dimensions(self):
-        return EmptyDict
-
-    def shape(self):
-        return ()
 
     def evaluate(self, x, mapping, component, index_values):
-        a = self._argument.evaluate(x, mapping, component, index_values)
+        a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
         try:
             res = getattr(math, self._name)(a)
         except ValueError:
@@ -82,11 +71,12 @@ class MathFunction(Operator):
         return res
 
     def __str__(self):
-        return "%s(%s)" % (self._name, self._argument)
+        return "%s(%s)" % (self._name, self.ufl_operands[0])
 
     def __repr__(self):
-        return "%s(%r)" % (self._name, self._argument)
+        return "%s(%r)" % (self._name, self.ufl_operands[0])
 
+@ufl_type()
 class Sqrt(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -97,6 +87,7 @@ class Sqrt(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sqrt", argument)
 
+@ufl_type()
 class Exp(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -107,6 +98,7 @@ class Exp(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "exp", argument)
 
+@ufl_type()
 class Ln(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -118,9 +110,10 @@ class Ln(MathFunction):
         MathFunction.__init__(self, "ln", argument)
 
     def evaluate(self, x, mapping, component, index_values):
-        a = self._argument.evaluate(x, mapping, component, index_values)
+        a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
         return math.log(a)
 
+@ufl_type()
 class Cos(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -131,6 +124,7 @@ class Cos(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "cos", argument)
 
+@ufl_type()
 class Sin(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -141,6 +135,7 @@ class Sin(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sin", argument)
 
+@ufl_type()
 class Tan(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -151,6 +146,7 @@ class Tan(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "tan", argument)
 
+@ufl_type()
 class Cosh(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -161,6 +157,7 @@ class Cosh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "cosh", argument)
 
+@ufl_type()
 class Sinh(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -171,6 +168,7 @@ class Sinh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "sinh", argument)
 
+@ufl_type()
 class Tanh(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -181,6 +179,7 @@ class Tanh(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "tanh", argument)
 
+@ufl_type()
 class Acos(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -191,6 +190,7 @@ class Acos(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "acos", argument)
 
+@ufl_type()
 class Asin(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -201,6 +201,7 @@ class Asin(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "asin", argument)
 
+@ufl_type()
 class Atan(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -211,48 +212,35 @@ class Atan(MathFunction):
     def __init__(self, argument):
         MathFunction.__init__(self, "atan", argument)
 
+@ufl_type(is_scalar=True, num_ops=2)
 class Atan2(Operator):
-    __slots__ = ("_name", "_arg1", "_arg2", "_classname")
+    __slots__ = ()
+
     def __new__(cls, arg1, arg2):
         if isinstance(arg1, (ScalarValue, Zero)) and isinstance(arg2, (ScalarValue, Zero)):
             return FloatValue(math.atan2(float(arg1), float(arg2)))
         return Operator.__new__(cls)
 
     def __init__(self, arg1, arg2):
-        Operator.__init__(self)
+        Operator.__init__(self, (arg1, arg2))
         ufl_assert(is_true_ufl_scalar(arg1), "Expecting scalar argument 1.")
         ufl_assert(is_true_ufl_scalar(arg2), "Expecting scalar argument 2.")
-        self._name     = "atan_2"
-        self._arg1 = arg1
-        self._arg2 = arg2
-
-    def operands(self):
-        return (self._arg1, self._arg2)
-
-    def free_indices(self):
-        return ()
-
-    def index_dimensions(self):
-        return EmptyDict
-
-    def shape(self):
-        return ()
 
     def evaluate(self, x, mapping, component, index_values):
-        a = self._arg1.evaluate(x, mapping, component, index_values)
-        b = self._arg2.evaluate(x, mapping, component, index_values)
+        a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
+        b = self.ufl_operands[1].evaluate(x, mapping, component, index_values)
         try:
-            res = math.atan2(a,b)
+            res = math.atan2(a, b)
         except ValueError:
-            warning('Value error in evaluation of function %s with arguments %s, %s.' % (self._name, a,b))
+            warning('Value error in evaluation of function atan_2 with arguments %s, %s.' % (a, b))
             raise
         return res
 
     def __str__(self):
-        return "%s(%s,%s)" % (self._name, self._arg1, self._arg2)
+        return "atan_2(%s,%s)" % (self.ufl_operands[0], self.ufl_operands[1])
 
     def __repr__(self):
-        return "%s(%s,%s)" % (self._name, self._arg1, self._arg2)
+        return "atan_2(%s,%s)" % (self.ufl_operands[0], self.ufl_operands[1])
 
 
 def _find_erf():
@@ -264,6 +252,7 @@ def _find_erf():
         return scipy.special.erf
     return None
 
+@ufl_type()
 class Erf(MathFunction):
     __slots__ = ()
     def __new__(cls, argument):
@@ -277,80 +266,75 @@ class Erf(MathFunction):
         MathFunction.__init__(self, "erf", argument)
 
     def evaluate(self, x, mapping, component, index_values):
-        a = self._argument.evaluate(x, mapping, component, index_values)
+        a = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
         erf = _find_erf()
         if erf is None:
             error("No python implementation of erf available on this system, cannot evaluate. Upgrade python or install scipy.")
         return erf(a)
 
+@ufl_type(is_abstract=True, is_scalar=True, num_ops=2)
 class BesselFunction(Operator):
     "Base class for all bessel functions"
     # Freeze member variables for objects in this class
-    __slots__ = ("_name", "_nu", "_argument", "_classname")
+    __slots__ = ("_name", "_classname")
     def __init__(self, name, classname, nu, argument):
-        Operator.__init__(self)
         ufl_assert(is_true_ufl_scalar(nu), "Expecting scalar nu.")
         ufl_assert(is_true_ufl_scalar(argument), "Expecting scalar argument.")
+
+        # Use integer representation if suitable
         fnu = float(nu)
         inu = int(nu)
         if fnu == inu:
             nu = as_ufl(inu)
         else:
             nu = as_ufl(fnu)
+
+        Operator.__init__(self, (nu, argument))
+
         self._classname = classname
         self._name     = name
-        self._nu       = nu
-        self._argument = argument
-
-    def operands(self):
-        return (self._nu, self._argument)
-
-    def free_indices(self):
-        return ()
-
-    def index_dimensions(self):
-        return EmptyDict
-
-    def shape(self):
-        return ()
 
     def evaluate(self, x, mapping, component, index_values):
-        a = self._argument.evaluate(x, mapping, component, index_values)
+        a = self.ufl_operands[1].evaluate(x, mapping, component, index_values)
         try:
             import scipy.special
         except:
             error("You must have scipy installed to evaluate bessel functions in python.")
         name = self._name[-1]
-        if isinstance(self._nu, IntValue):
-            nu = int(self._nu)
+        if isinstance(self.ufl_operands[0], IntValue):
+            nu = int(self.ufl_operands[0])
             functype = 'n' if name != 'i' else 'v'
         else:
-            nu = self._nu.evaluate(x, mapping, component, index_values)
+            nu = self.ufl_operands[0].evaluate(x, mapping, component, index_values)
             functype = 'v'
         func = getattr(scipy.special, name + functype)
         return func(nu, a)
 
     def __str__(self):
-        return "%s(%s, %s)" % (self._name, self._nu, self._argument)
+        return "%s(%s, %s)" % (self._name, self.ufl_operands[0], self.ufl_operands[1])
 
     def __repr__(self):
-        return "%s(%r, %r)" % (self._classname, self._nu, self._argument)
+        return "%s(%r, %r)" % (self._classname, self.ufl_operands[0], self.ufl_operands[1])
 
+@ufl_type()
 class BesselJ(BesselFunction):
     __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_j", "BesselJ", nu, argument)
 
+@ufl_type()
 class BesselY(BesselFunction):
     __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_y", "BesselY", nu, argument)
 
+@ufl_type()
 class BesselI(BesselFunction):
     __slots__ = ()
     def __init__(self, nu, argument):
         BesselFunction.__init__(self, "cyl_bessel_i", "BesselI", nu, argument)
 
+@ufl_type()
 class BesselK(BesselFunction):
     __slots__ = ()
     def __init__(self, nu, argument):
