@@ -21,7 +21,7 @@
 
 from ufl.assertions import ufl_assert
 from ufl.log import error, warning
-from ufl.expr import Expr
+from ufl.core.expr import Expr
 from ufl.geometry import Domain, as_domain
 from ufl.checks import is_true_ufl_scalar
 from ufl.constantvalue import as_ufl
@@ -35,27 +35,34 @@ from ufl.protocols import id_or_none, metadata_equal, metadata_hashdata
 # Enumeration of valid domain types
 _integral_types = [
     # === Integration over full topological dimension:
-    ("cell", "dx"),                # Over a single cell
+    ("cell", "dx"),                     # Over cells of a mesh
+    #("macro_cell", "dE"),              # Over a group of adjacent cells (TODO: Arbitrary cell group? Not currently used.)
+
     # === Integration over topological dimension - 1:
-    ("exterior_facet", "ds"),           # Over facet of a single cell
+    ("exterior_facet", "ds"),           # Over one-sided exterior facets of a mesh
+    ("interior_facet", "dS"),           # Over two-sided facets between pairs of adjacent cells of a mesh
+
+    # === Integration over topological dimension 0
+    ("vertex", "dP"),                   # Over vertices of a mesh
+    #("vertex", "dV"),                  # TODO: Use this over vertices?
+    #("point", "dP"),                   # TODO: Use this over arbitrary points inside cells?
+
+    # === Integration over custom domains
+    ("custom", "dc"),                   # Over custom user-defined domains (run-time quadrature points)
+    ("overlap", "dO"),                  # Over a cell fragment overlapping with two or more cells (run-time quadrature points)
+    ("interface", "dI"),                # Over facet fragment overlapping with two or more cells (run-time quadrature points)
+    ("cutcell", "dC"),                  # Over a cell with some part cut away (run-time quadrature points)
+
+    # === Firedrake specific hacks on the way out:
+    # TODO: Remove these, firedrake can use metadata instead and create the measure objects in firedrake:
     ("exterior_facet_bottom", "ds_b"),  # Over bottom facets on extruded mesh
     ("exterior_facet_top", "ds_t"),     # Over top facets on extruded mesh
     ("exterior_facet_vert", "ds_v"),    # Over side facets of an extruded mesh
-    ("interior_facet", "dS"),           # Over facet between two adjacent cells
     ("interior_facet_horiz", "dS_h"),   # Over horizontal facets of an extruded mesh
     ("interior_facet_vert", "dS_v"),    # Over vertical facets of an extruded mesh
-    # === Integration over topological dimension 0
-    ("point", "dP"),               # TODO: Is this over arbitrary point cloud or vertices?
-    # === Integration over custom domains
-    ("custom", "dc"),              # Over custom user-defined domains (defined in terms of quadrature points)
-    # === Other types (up for discussion, unclear whether they are used)
-    ("macro_cell", "dE"),          # Over a group of adjacent cells (TODO: Arbitrary cell group? Where is this used?)
-    #("overlap", "dO"),            # TODO: Over a cell fragment overlapping with two or more cells
-    #("interface", "dI"),          # Over facet fragment overlapping with two or more cells
-    #("vertex", "dV"),             # TODO: Use this over vertices?
     ]
-integral_type_to_measure_name = dict((l,s) for l,s in _integral_types)
-measure_name_to_integral_type = dict((s,l) for l,s in _integral_types)
+integral_type_to_measure_name = dict((l, s) for l, s in _integral_types)
+measure_name_to_integral_type = dict((s, l) for l, s in _integral_types)
 
 def register_integral_type(integral_type, measure_name):
     global integral_type_to_measure_name, measure_name_to_integral_type
@@ -96,20 +103,6 @@ class Measure(object):
     transferred to a Form on multiplication with a scalar expression.
     """
 
-    # Enumeration of valid domain types (TODO: Remove these)
-    CELL           = "cell"
-    EXTERIOR_FACET = "exterior_facet"
-    EXTERIOR_FACET_BOTTOM = "exterior_facet_bottom"
-    EXTERIOR_FACET_TOP = "exterior_facet_top"
-    EXTERIOR_FACET_VERT = "exterior_facet_vert"
-    INTERIOR_FACET = "interior_facet"
-    INTERIOR_FACET_HORIZ = "interior_facet_horiz"
-    INTERIOR_FACET_VERT = "interior_facet_vert"
-    POINT          = "point"
-    QUADRATURE     = "quadrature"
-    MACRO_CELL     = "macro_cell"
-    SURFACE        = "surface"
-
     def __init__(self,
                  integral_type, # "dx" etc
                  domain=None,
@@ -223,14 +216,14 @@ class Measure(object):
         Its intension is to give a context in which the domain id is interpreted."""
         return self._subdomain_data
 
-    def __call__(self, subdomain_id=None, metadata=None, domain=None, subdomain_data=None):
+    # Note: Must keep the order of the first two arguments here (subdomain_id, metadata) for
+    # backwards compatibility, because some tutorials write e.g. dx(0, {...}) to set metadata.
+    def __call__(self, subdomain_id=None, metadata=None, domain=None, subdomain_data=None, degree=None, rule=None):
         """Reconfigure measure with new domain specification or metadata."""
-        # Note: Keeping the order of arguments here (subdomain_id, metadata) for backwards
-        # compatibility, because some tutorials write e.g. dx(0, {...}) to set metadata
 
         # Let syntax dx() mean integral over everywhere
-        args = (subdomain_id, metadata, domain, subdomain_data)
-        if all(arg is None for arg in args):
+        all_args = (subdomain_id, metadata, domain, subdomain_data, degree, rule)
+        if all(arg is None for arg in all_args):
             return self.reconstruct(subdomain_id="everywhere")
 
         # Let syntax dx(domain) or dx(domain, metadata) mean integral over entire domain.
@@ -239,6 +232,15 @@ class Measure(object):
             ufl_assert(domain is None, "Ambiguous: setting domain both as keyword argument and first argument.")
             subdomain_id, domain = "everywhere", as_domain(subdomain_id)
 
+        # If degree or rule is set, inject into metadata. This is a quick fix to enable
+        # the dx(..., degree=3) notation. TODO: Make degree and rule properties of integrals.
+        if (degree, rule) != (None, None):
+            metadata = {} if metadata is None else metadata.copy()
+            if degree is not None:
+                metadata["quadrature_degree"] = degree
+            if rule is not None:
+                metadata["quadrature_rule"] = rule
+
         # If we get any keywords, use them to reconstruct Measure.
         # Note that if only one argument is given, it is the subdomain_id,
         # e.g. dx(3) == dx(subdomain_id=3)
@@ -357,7 +359,7 @@ class Measure(object):
         from ufl.form import Form
 
         # Allow python literals: 1*dx and 1.0*dx
-        if isinstance(integrand, (int,float)):
+        if isinstance(integrand, (int, float)):
             integrand = as_ufl(integrand)
 
         # Let other types implement multiplication with Measure
@@ -369,7 +371,7 @@ class Measure(object):
         if not is_true_ufl_scalar(integrand):
             msg = ("Can only integrate scalar expressions. The integrand is a " +
                    "tensor expression with value rank %d and free indices %r.")
-            error(msg % (integrand.rank(), integrand.free_indices()))
+            error(msg % (integrand.rank(), integrand.ufl_free_indices))
 
         # If we have a tuple of domain ids, delegate composition to Integral.__add__:
         subdomain_id = self.subdomain_id()
@@ -378,7 +380,7 @@ class Measure(object):
 
         # Check that we have an integer subdomain or a string
         # ("everywhere" or "otherwise", any more?)
-        ufl_assert(isinstance(subdomain_id, (int,str)),
+        ufl_assert(isinstance(subdomain_id, (int, str)),
                    "Expecting integer or string domain id.")
 
         # If we don't have an integration domain, try to find one in integrand
@@ -443,7 +445,7 @@ class MeasureSum(object):
         return NotImplemented
 
     def __str__(self):
-        return "{\n    " + "\n  + ".join(map(str,self._measures)) + "\n}"
+        return "{\n    " + "\n  + ".join(map(str, self._measures)) + "\n}"
 
 class MeasureProduct(object):
     """Represents a product of measures.
