@@ -19,36 +19,22 @@
 #
 # Modified by Anders Logg, 2008
 # Modified by Kristian Oelgaard, 2009
-#
-# First added:  2008-03-14
-# Last changed: 2014-03-19
 
-from ufl.indexing import indices
-from ufl.integral import Measure
-from ufl.geometry import Cell
+from ufl.core.multiindex import indices
+from ufl.cell import Cell
+from ufl.measure import Measure
 
 # Default indices
 i, j, k, l = indices(4)
 p, q, r, s = indices(4)
 
 # Default measures for integration
-dx = Measure("cell")
-ds = Measure("exterior_facet")
-ds_b = Measure("exterior_facet_bottom")
-ds_t = Measure("exterior_facet_top")
-ds_v = Measure("exterior_facet_vert")
-ds_tb = ds_b + ds_t
-dS = Measure("interior_facet")
-dS_h = Measure("interior_facet_horiz")
-dS_v = Measure("interior_facet_vert")
-dP = Measure("point")
-dc = Measure("custom")
-dE = Measure("macro_cell")
+from ufl.measure import integral_type_to_measure_name
+for integral_type, measure_name in integral_type_to_measure_name.items():
+    globals()[measure_name] = Measure(integral_type)
+ds_tb = ds_b + ds_t # TODO: Firedrake hack, remove later
 
 # Cell types
-cell1D        = Cell("cell1D", 1)
-cell2D        = Cell("cell2D", 2)
-cell3D        = Cell("cell3D", 3)
 vertex        = Cell("vertex", 0)
 interval      = Cell("interval", 1)
 triangle      = Cell("triangle", 2)
