@@ -1,6 +1,6 @@
 """Restriction operations."""
 
-# Copyright (C) 2008-2014 Martin Sandve Alnes
+# Copyright (C) 2008-2015 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -16,123 +16,101 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2008-06-08
-# Last changed: 2011-06-02
 
 from ufl.log import error
-from ufl.operatorbase import Operator
+from ufl.core.operator import Operator
 from ufl.precedence import parstr
-from ufl.common import EmptyDict
+from ufl.core.ufl_type import ufl_type
 
 #--- Restriction operators ---
 
+@ufl_type(is_abstract=True,
+          num_ops=1,
+          inherit_shape_from_operand=0,
+          inherit_indices_from_operand=0,
+          is_restriction=True)
 class Restricted(Operator):
-    __slots__ = ("_f", "_side")
+    __slots__ = ()
 
     # TODO: Add __new__ operator here, e.g. restricted(literal) == literal
 
-    def __init__(self, f, side):
-        Operator.__init__(self)
-        self._f = f
-        self._side = side
+    def __init__(self, f):
+        Operator.__init__(self, (f,))
 
-    def shape(self):
-        return self._f.shape()
-
-    def operands(self):
-        return (self._f,)
-
-    def free_indices(self):
-        return self._f.free_indices()
-
-    def index_dimensions(self):
-        return self._f.index_dimensions()
+    def side(self):
+        return self._side
 
     def evaluate(self, x, mapping, component, index_values):
-        return self._f.evaluate(x, mapping, component, index_values)
+        return self.ufl_operands[0].evaluate(x, mapping, component, index_values)
 
     def __str__(self):
-        return "%s('%s')" % (parstr(self._f, self), self._side)
+        return "%s('%s')" % (parstr(self.ufl_operands[0], self), self._side)
 
+    def __repr__(self):
+        return "%s(%r)" % (self._ufl_class_.__name__, self.ufl_operands[0])
+
+@ufl_type(is_terminal_modifier=True)
 class PositiveRestricted(Restricted):
     __slots__ = ()
-    def __init__(self, f):
-        Restricted.__init__(self, f, "+")
-
-    def __repr__(self):
-        return "PositiveRestricted(%r)" % self._f
+    _side = "+"
 
+@ufl_type(is_terminal_modifier=True)
 class NegativeRestricted(Restricted):
     __slots__ = ()
-    def __init__(self, f):
-        Restricted.__init__(self, f, "-")
-
-    def __repr__(self):
-        return "NegativeRestricted(%r)" % self._f
+    _side = "-"
 
 
 # TODO: Place in a better file?
+@ufl_type(is_index_free=True,
+          num_ops=1,
+          is_terminal_modifier=True,
+          is_evaluation=True)
 class CellAvg(Operator):
-    __slots__ = ("_f",)
+    __slots__ = ()
 
     # TODO: Add __new__ operator here, e.g. cell_avg(literal) == literal
 
     def __init__(self, f):
-        Operator.__init__(self)
-        self._f = f
+        Operator.__init__(self, (f,))
 
-    def shape(self):
-        return self._f.shape()
-
-    def operands(self):
-        return (self._f,)
-
-    def free_indices(self):
-        return ()
-
-    def index_dimensions(self):
-        return EmptyDict
+    @property
+    def ufl_shape(self):
+        return self.ufl_operands[0].ufl_shape
 
     def evaluate(self, x, mapping, component, index_values):
         "Performs an approximate symbolic evaluation, since we dont have a cell."
-        return self._f.evaluate(x, mapping, component, index_values)
+        return self.ufl_operands[0].evaluate(x, mapping, component, index_values)
 
     def __str__(self):
-        return "cell_avg(%s)" % (self._f,)
+        return "cell_avg(%s)" % (self.ufl_operands[0],)
 
     def __repr__(self):
-        return "CellAvg(%r)" % self._f
+        return "CellAvg(%r)" % self.ufl_operands[0]
 
 
 # TODO: Place in a better file?
+@ufl_type(is_index_free=True,
+          num_ops=1,
+          is_terminal_modifier=True,
+          is_evaluation=True)
 class FacetAvg(Operator):
-    __slots__ = ("_f",)
+    __slots__ = ()
 
     # TODO: Add __new__ operator here, e.g. facet_avg(literal) == literal
 
     def __init__(self, f):
-        Operator.__init__(self)
-        self._f = f
+        Operator.__init__(self, (f,))
 
-    def shape(self):
-        return self._f.shape()
-
-    def operands(self):
-        return (self._f,)
-
-    def free_indices(self):
-        return ()
-
-    def index_dimensions(self):
-        return EmptyDict
+    @property
+    def ufl_shape(self):
+        return self.ufl_operands[0].ufl_shape
 
     def evaluate(self, x, mapping, component, index_values):
         "Performs an approximate symbolic evaluation, since we dont have a cell."
-        return self._f.evaluate(x, mapping, component, index_values)
+        return self.ufl_operands[0].evaluate(x, mapping, component, index_values)
 
     def __str__(self):
-        return "facet_avg(%s)" % (self._f,)
+        return "facet_avg(%s)" % (self.ufl_operands[0],)
 
     def __repr__(self):
-        return "FacetAvg(%r)" % self._f
+        return "FacetAvg(%r)" % self.ufl_operands[0]
